from ctypes import byref, c_byte, c_int16, c_int32, sizeof
from time import sleep

from picosdk.ps2000 import ps2000
from picosdk.functions import assert_pico2000_ok, adc2mV
from picosdk.PicoDeviceEnums import picoEnum

import matplotlib.pyplot as plt


SAMPLES = 2000
OVERSAMPLING = 1


def get_timebase(device, wanted_time_interval):
    current_timebase = 1

    old_time_interval = None
    time_interval = c_int32(0)
    time_units = c_int16()
    max_samples = c_int32()

    while ps2000.ps2000_get_timebase(
        device.handle,
        current_timebase,
        2000,
        byref(time_interval),
        byref(time_units),
        1,
        byref(max_samples)) == 0 \
        or time_interval.value < wanted_time_interval:

        current_timebase += 1
        old_time_interval = time_interval.value

        if current_timebase.bit_length() > sizeof(c_int16) * 8:
            raise Exception('No appropriate timebase was identifiable')

    return current_timebase - 1, old_time_interval


with ps2000.open_unit() as device:
    print('Device info: {}'.format(device.info))

    res = ps2000.ps2000_set_channel(
        device.handle,
        picoEnum.PICO_CHANNEL['PICO_CHANNEL_A'],
        True,
        picoEnum.PICO_COUPLING['PICO_DC'],
        ps2000.PS2000_VOLTAGE_RANGE['PS2000_500MV'],
    )
    assert_pico2000_ok(res)

    res = ps2000.ps2000_set_channel(
        device.handle,
        picoEnum.PICO_CHANNEL['PICO_CHANNEL_B'],
        True,
        picoEnum.PICO_COUPLING['PICO_DC'],
        ps2000.PS2000_VOLTAGE_RANGE['PS2000_50MV'],
    )
    assert_pico2000_ok(res)

    timebase_a, interval = get_timebase(device, 4_000)

    collection_time = c_int32()

    res = ps2000.ps2000_run_block(
        device.handle,
        SAMPLES,
        timebase_a,
        OVERSAMPLING,
        byref(collection_time)
    )
    assert_pico2000_ok(res)

    while ps2000.ps2000_ready(device.handle) == 0:
        sleep(0.1)

    times = (c_int32 * SAMPLES)()

    buffer_a = (c_int16 * SAMPLES)()
    buffer_b = (c_int16 * SAMPLES)()

    overflow = c_byte(0)

    res = ps2000.ps2000_get_times_and_values(
        device.handle,
        byref(times),
        byref(buffer_a),
        byref(buffer_b),
        None,
        None,
        byref(overflow),
        2,
        SAMPLES,
    )
    assert_pico2000_ok(res)

    channel_a_overflow = (overflow.value & 0b0000_0001) != 0

    ps2000.ps2000_stop(device.handle)

    channel_a_mv = adc2mV(buffer_a, ps2000.PS2000_VOLTAGE_RANGE['PS2000_500MV'], c_int16(32767))
    channel_b_mv = adc2mV(buffer_b, ps2000.PS2000_VOLTAGE_RANGE['PS2000_50MV'], c_int16(32767))

    fig, ax = plt.subplots()
    ax.set_xlabel('time/ms')
    ax.set_ylabel('voltage/mV')
    ax.plot(list(map(lambda x: x * 1e-6, times[:])), channel_a_mv[:])
    ax.plot(list(map(lambda x: x * 1e-6, times[:])), channel_b_mv[:])

    if channel_a_overflow:
        ax.text(0.01, 0.01, 'Overflow present', color='red', transform=ax.transAxes)

    plt.show()



#
# Copyright (C) 2018 Pico Technology Ltd. See LICENSE file for terms.
#
# PicoScope 3000 (A API) Series Signal Generator Example
# This example demonstrates how to use the PicoScope 3000 Series (ps3000a) driver API functions to set up the signal generator to do the following:
# 
# 1. Output a sine wave 
# 2. Output a square wave 
# 3. Output a sweep of a square wave signal

import ctypes
from picosdk.ps3000a import ps3000a as ps
import time
from picosdk.functions import assert_pico_ok


# Gives the device a handle
status = {}
chandle = ctypes.c_int16()

# Opens the device/s
status["openunit"] = ps.ps3000aOpenUnit(ctypes.byref(chandle), None)

try:
    assert_pico_ok(status["openunit"])
except:

    # powerstate becomes the status number of openunit
    powerstate = status["openunit"]

    # If powerstate is the same as 282 then it will run this if statement
    if powerstate == 282:
        # Changes the power input to "PICO_POWER_SUPPLY_NOT_CONNECTED"
        status["ChangePowerSource"] = ps.ps3000aChangePowerSource(chandle, 282)
    # If the powerstate is the same as 286 then it will run this if statement
    elif powerstate == 286:
        # Changes the power input to "PICO_USB3_0_DEVICE_NON_USB3_0_PORT"
        status["ChangePowerSource"] = ps.ps3000aChangePowerSource(chandle, 286)
    else:
        raise

    assert_pico_ok(status["ChangePowerSource"])

# Output a sine wave with peak-to-peak voltage of 2 V and frequency of 10 kHz
# handle = chandle
# offsetVoltage = 0
# pkToPk = 2000000
# waveType = ctypes.c_int16(0) = PS3000A_SINE
# startFrequency = 10000 Hz
# stopFrequency = 10000 Hz
# increment = 0
# dwellTime = 1
# sweepType = ctypes.c_int16(1) = PS3000A_UP
# operation = 0
# shots = 0
# sweeps = 0
# triggerType = ctypes.c_int16(0) = PS3000A_SIGGEN_RISING
# triggerSource = ctypes.c_int16(0) = P3000A_SIGGEN_NONE
# extInThreshold = 1
wavetype = ctypes.c_int16(0)
sweepType = ctypes.c_int32(0)
triggertype = ctypes.c_int32(0)
triggerSource = ctypes.c_int32(0)

status["SetSigGenBuiltIn"] = ps.ps3000aSetSigGenBuiltIn(chandle, 0, 2000000, wavetype, 10000, 10000, 0, 1, sweepType, 0, 0, 0, triggertype, triggerSource, 1)
assert_pico_ok(status["SetSigGenBuiltIn"])

# Pauses the script to show signal
time.sleep(10)

# Output a square wave with peak-to-peak voltage of 2 V and frequency of 10 kHz
# handle = chandle
# offsetVoltage = -1000000
# pkToPk = 1500000
# waveType = ctypes.c_int16(1) = PS3000A_SQUARE
# startFrequency = 10 kHz
# stopFrequency = 10 kHz
# increment = 0
# dwellTime = 1
# sweepType = ctypes.c_int16(1) = PS3000A_UP
# operation = 0
# shots = 0
# sweeps = 0
# triggerType = ctypes.c_int16(0) = PS3000A_SIGGEN_RISING
# triggerSource = ctypes.c_int16(0) = P3000A_SIGGEN_NONE
# extInThreshold = 1
wavetype = ctypes.c_int16(1)
sweepType = ctypes.c_int32(0)
triggertype = ctypes.c_int32(0)
triggerSource = ctypes.c_int32(0)

status["SetSigGenBuiltIn"] = ps.ps3000aSetSigGenBuiltIn(chandle, 0, 2000000, wavetype, 10000, 10000, 0, 1, sweepType, 0, 0, 0, triggertype, triggerSource, 1)
assert_pico_ok(status["SetSigGenBuiltIn"])

# Pauses the script to show signal
time.sleep(10)

# Output a square wave with an up-down sweep, 10-100 kHz in 5 kHz increments every 1 second.
# handle = chandle
# offsetVoltage = -1000000
# pkToPk = 1500000
# waveType = ctypes.c_int16(1) = PS3000A_Square
# startFrequency = 100 kHz
# stopFrequency = 100 kHz
# increment = 5 kHz
# dwellTime = 1
# sweepType = ctypes.c_int16(1) = PS3000A_UP
# operation = 0
# shots = 0
# sweeps = 0
# triggerType = ctypes.c_int16(0) = PS3000A_SIGGEN_RISING
# triggerSource = ctypes.c_int16(0) = P3000A_SIGGEN_NONE
# extInThreshold = 1
wavetype = ctypes.c_int16(1)
sweepType = ctypes.c_int32(2)
triggertype = ctypes.c_int32(0)
triggerSource = ctypes.c_int32(0)

status["SetSigGenBuiltIn"] = ps.ps3000aSetSigGenBuiltIn(chandle, 0, 2000000, wavetype, 10000, 100000, 5000, 1, sweepType, 0, 0, 0, triggertype, triggerSource, 1)
assert_pico_ok(status["SetSigGenBuiltIn"])

# Pauses the script to show signal
time.sleep(36)

# Closes the unit
# Handle = chandle
status["close"] = ps.ps3000aCloseUnit(chandle)
assert_pico_ok(status["close"])

# Displays the status returns
print(status)



#
# Copyright (C) 2019 Pico Technology Ltd. See LICENSE file for terms.
#
"""
This is a Python module defining the functions from the pl1000.h C header
file for PicoLog 1000 Series datalogger using the pl1000 driver API functions.
"""

from ctypes import *
from picosdk.ctypes_wrapper import C_CALLBACK_FUNCTION_FACTORY
from picosdk.library import Library
from picosdk.constants import make_enum

class Pl1000lib(Library):
    def __init__(self):
        super(Pl1000lib, self).__init__("pl1000")
		
pl1000 = Pl1000lib()

def _pl1000Inputs():
    PL1000_CHANNEL_1 = 1
    PL1000_CHANNEL_2 = 2
    PL1000_CHANNEL_3 = 3
    PL1000_CHANNEL_4 = 4
    PL1000_CHANNEL_5 = 5
    PL1000_CHANNEL_6 = 6
    PL1000_CHANNEL_7 = 7
    PL1000_CHANNEL_8 = 8
    PL1000_CHANNEL_9 = 9
    PL1000_CHANNEL_10 = 10
    PL1000_CHANNEL_11 = 11
    PL1000_CHANNEL_12 = 12
    PL1000_CHANNEL_13 = 13
    PL1000_CHANNEL_14 = 14
    PL1000_CHANNEL_15 = 15
    PL1000_CHANNEL_16 = 16
    PL1000_MAX_CHANNEL = PL1000_CHANNEL_16
	
    return {k.upper(): v for k, v in locals().items() if k.startswith("PL1000")}
	
pl1000.PL1000Inputs = _pl1000Inputs()

pl1000.PL1000DO_Channel = make_enum([
    'PL1000_DO_CHANNEL_0',
	'PL1000_DO_CHANNEL_1',
	'PL1000_DO_CHANNEL_2',
	'PL1000_DO_CHANNEL_3',
	'PL1000_DO_CHANNEL_MAX',
])

pl1000.PL1000OpenProgress = {
    'PL1000_OPEN_PROGRESS_FAIL' : -1,
	'PL1000_OPEN_PROGRESS_PENDING': 0,
	'PL1000_OPEN_PROGRESS_COMPLETE' : 1,
}

pl1000.PL1000_BLOCK_METHOD = make_enum([
	"BM_SINGLE",
	"BM_WINDOW",
	"BM_STREAM",
])

doc = """ PICO_STATUS pl1000CloseUnit
    (
	    int16_t  handle
	); """
pl1000.make_symbol("_CloseUnit_", "pl1000CloseUnit", c_uint32, [c_int16], doc)

doc = """ PICO_STATUS pl1000GetSingle
    (
	    int16_t  handle,
		PL1000_INPUTS  channel,
		unit16_t  *value
	); """
pl1000.make_symbol("_GetSingle_", "pl1000GetSingle", c_uint32, [c_int16, c_int32, c_void_p], doc)

doc = """ PICO_STATUS pl1000GetUnitInfo
    (
	    int16_t  handle,
		int8_t  *string,
		int16_t  stringLength,
		int16_t  *requiredSize,
		PICO_INFO  info
	); """
pl1000.make_symbol("_GetUnitInfo_", "pl1000GetUnitInfo", c_uint32, [c_int16, c_void_p, c_int16, c_void_p, c_int32], doc)

doc = """ PICO_STATUS pl1000GetValues
    (
	    int16_t  handle,
		uint16_t  *values,
		uint32_t  *noOfValues,
		unit16_t  *overflow,
		uint32_t  *triggerIndex
	); """
pl1000.make_symbol("_GetValues_", "pl1000GetValues", c_uint32, [c_int16, c_void_p, c_void_p, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS pl1000MaxValue
    (
	    int16_t  handle,
		uint16_t  *maxValue
	); """
pl1000.make_symbol("_MaxValue_", "pl1000MaxValue", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS pl1000OpenUnit
    (
	    int16_t  *handle
	); """
pl1000.make_symbol("_OpenUnit_", "pl1000OpenUnit", c_uint32, [c_void_p], doc)

doc = """ PICO_STATUS pl1000OpenUnitAsync
    (
	    int16_t  *status
	); """
pl1000.make_symbol("_OpenUnitAsync_", "pl1000OpenUnitAsync", c_uint32, [c_void_p], doc)

doc = """ PICO_STATUS pl1000OpenUnitProgress
    (
	    int16_t  *handle,
		int16_t  *progress,
		int16_t  *complete
	); """
pl1000.make_symbol("_OpenUnitProgress_", "pl1000OpenUnitProgress", c_uint32, [c_void_p, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS pl1000PingUnit
    (
	    int16_t  handle
	); """
pl1000.make_symbol("_PingUnit_", "pl1000PingUnit", c_uint32, [c_int16], doc)

doc = """ PICO_STATUS pl1000Ready
    (
	    int16_t  handle,
		int16_t  *ready
	); """
pl1000.make_symbol("_Ready_", "pl1000Ready", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS pl1000Run
    (
	    int16_t  handle,
		uint32_t  no_of_values,
		BLOCK_METHOD  method
	); """
pl1000.make_symbol("_Run_", "pl1000Run", c_uint32, [c_int16, c_uint32, c_int32], doc)

doc = """ PICO_STATUS pl1000SetDo
    (
	    int16_t  handle,
		int16_t  do_value,
		int16_t  doNo
	); """
pl1000.make_symbol("_SetDo_", "pl1000SetDo", c_uint32, [c_int16, c_int16, c_int16], doc)

doc = """ PICO_STATUS pl1000SetInterval
    (
	    int16_t  handle,
		uint32_t  *us_for_block,
		uint32_t  ideal_no_of_samples,
		int16_t  *channels,
		int16_t  no_of_channels
	); """
pl1000.make_symbol("_SetInterval_", "pl1000SetInterval", c_uint32, [c_int16, c_void_p, c_uint32, c_void_p, c_int16], doc)

doc = """ PICO_STATUS pl1000SetPulseWidth
    (
	    int16_t  handle,
		uint16_t  period,
		uint8_t  cycle
	); """
pl1000.make_symbol("_SetPulseWidth_", "pl1000SetPulseWidth", c_uint32, [c_int16, c_int16, c_int8], doc)

doc = """ PICO_STATUS pl1000SetTrigger
    (
	    int16_t  handle,
		uint16_t  enabled,
		uint16_t  auto_trigger,
		uint16_t  auto_ms,
		uint16_t  channel,
		uint16_t  dir,
		uint16_t  threshold,
		uint16_t  hysteresis,
		float  delay
	); """
pl1000.make_symbol("_SetTrigger_", "pl1000SetTrigger", c_uint32, [c_int16, c_uint16, c_uint16, c_uint16, c_uint16, c_uint16, c_uint16, c_uint16, c_float], doc)

doc = """ PICO_STATUS pl1000Stop
    (
	    int16_t  handle
	); """
pl1000.make_symbol("_Stop_", "pl1000Stop", c_uint32, [c_int16], doc)




#
# Copyright (C) 2020 Pico Technology Ltd. See LICENSE file for terms.
#
"""
This is a Python module defining the functions from the ps6000aApi.h C header
file for PicoScope 6000 A Series oscilloscopes using the ps6000a driver API
functions.
"""

from ctypes import *
from picosdk.library import Library
from picosdk.ctypes_wrapper import C_CALLBACK_FUNCTION_FACTORY
from picosdk.constants import make_enum


class Ps6000alib(Library):
    def __init__(self):
        super(Ps6000alib, self).__init__("ps6000a")

ps6000a = Ps6000alib()

doc = """ void ps6000aBlockReady
    (
        int16_t    handle,
        PICO_STATUS    status,
        PICO_POINTER    pParameter
    ); """
ps6000a.BlockReadyType = C_CALLBACK_FUNCTION_FACTORY(None,
                                                     c_int16,
                                                     c_uint32,
                                                     c_void_p)
ps6000a.BlockReadyType.__doc__ = doc

doc = """ void ps6000aStreamingReady
    (
        int16_t    handle,
        int64_t    noOfSamples,
        uint64_t    bufferIndex,
        uint32_t    startIndex,
        int16_t    overflow,
        uint32_t    triggerAt,
        int16_t    triggered,
        int16_t    autoStop,
        PICO_POINTER    pParameter
    ); """
ps6000a.StreamingReadyType = C_CALLBACK_FUNCTION_FACTORY(None,
                                                         c_int16,
                                                         c_int64,
                                                         c_uint64,
                                                         c_uint32,
                                                         c_int16,
                                                         c_uint32,
                                                         c_int16,
                                                         c_int16,
                                                         c_void_p)
ps6000a.StreamingReadyType.__doc__ = doc

doc = """ void ps6000aDataReady
    (
        int16_t    handle,
        PICO_STATUS    status,
        uint64_t    noOfSamples,
        int16_t    overflow,
        PICO_POINTER    pParameter
    ); """
ps6000a.DataReadyType = C_CALLBACK_FUNCTION_FACTORY(None,
                                                    c_int16,
                                                    c_uint32,
                                                    c_uint64,
                                                    c_int16,
                                                    c_void_p)
ps6000a.DataReadyType.__doc__ = doc

doc = """ void ps6000aProbeInteractions
    (
        int16_t    handle,
        PICO_STATUS    status,
        PICO_USER_PROBE_INTERACTIONS    *probes,
        uint32_t    nProbes
    ); """
ps6000a.ProbeInteractionsType = C_CALLBACK_FUNCTION_FACTORY(None,
                                                            c_int16,
                                                            c_uint32,
                                                            c_void_p,
                                                            c_uint32)
ps6000a.ProbeInteractionsType.__doc__ = doc

doc = """ void ps6000aDigitalPortInteractions
    (
        int16_t    handle,
        PICO_STATUS    status,
        PICO_DIGITAL_PORT_INTERACTIONS    *ports,
        uint32_t    nPorts
    ); """
ps6000a.DigitalPortInteractionsType = C_CALLBACK_FUNCTION_FACTORY(None,
                                                                  c_int16,
                                                                  c_uint32,
                                                                  c_void_p,
                                                                  c_uint32)
ps6000a.DigitalPortInteractionsType.__doc__ = doc

doc = """ PICO_STATUS ps6000aOpenUnit
    (
        int16_t *handle,
        int8_t  *serial
        PICO_DEVICE_RESOLUTION    resolution
    ); """
ps6000a.make_symbol("_OpenUnit", "ps6000aOpenUnit", c_uint32, [c_void_p, c_char_p, c_int32], doc)

doc = """ PICO_STATUS ps6000aOpenUnitAsync
    (
        int16_t    *handle,
        int8_t    *serial,
        PICO_DEVICE_RESOLUTION    resolution
    ); """
ps6000a.make_symbol("_OpenUnitAsync", "ps6000aOpenUnitAsync", c_uint32, [c_void_p, c_char_p, c_int32], doc)

doc = """ PICO_STATUS ps6000aOpenUnitProgress
    (
        int16_t    *handle,
        int16_t    *progressPercent,
        int16_t    *complete
    ); """
ps6000a.make_symbol("_OpenUnitProgress", "ps6000aOpenUnitProgress", c_uint32, [c_void_p, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps6000aGetUnitInfo
    (
        int16_t    handle,
        int8_t    *string,
        int16_t    stringLength,
        int16_t    *requiredSize,
        PICO_INFO    info
    ); """
ps6000a.make_symbol("_GetUnitInfo", "ps6000aGetUnitInfo", c_uint32, [c_int16, c_char_p, c_int16, c_void_p, c_int32], doc)

doc = """ PICO_STATUS ps6000aCloseUnit
    (
        int16_t    handle
    ); """
ps6000a.make_symbol("_CloseUnit", "ps6000aCloseUnit", c_uint32, [c_int16], doc)

doc = """ PICO_STATUS ps6000aFlashLed
    (
        int16_t    handle,
        int16_t    start
    ); """
ps6000a.make_symbol("_FlashLed", "ps6000aFlashLed", c_uint32, [c_int16, c_int16], doc)

doc = """ PICO_STATUS ps6000aMemorySegments
    (
        int16_t    handle,
        uint64_t    nSegments,
        uint64_t    *nMaxSegments
    ); """
ps6000a.make_symbol("_MemorySegments", "ps6000aMemorySegments", c_uint32, [c_int16, c_uint64, c_void_p], doc)

doc = """ PICO_STATUS ps6000aMemorySegmentsBySamples
    (
        int16_t    handle,
        uint64_t    nSamples,
        uint64_t    *nMaxSegments
    ); """
ps6000a.make_symbol("_MemorySegmentsBySamples", "ps6000aMemorySegmentsBySamples", c_uint32, [c_int16, c_uint64, c_void_p], doc)

doc = """ PICO_STATUS ps6000aGetMaximumAvailableMemory
    (
        int16_t    handle,
        uint64_t    *nMaxSamples,
        PICO_DEVICE_RESOLUTION    resolution
    ); """
ps6000a.make_symbol("_GetMaximumAvailableMemory", "ps6000aGetMaximumAvailableMemory", c_uint32, [c_int16, c_void_p, c_uint32], doc)

doc = """ PICO_STATUS ps6000aQueryMaxSegmentsBySamples
    (
        int16_t    handle,
        uint64_t    nSamples,
        uint32_t    nChannelsEnabled,
        uint64_t    *nMaxSegments,
        PICO_DEVICE_RESOLUTION    resolution
    ); """
ps6000a.make_symbol("_QueryMaxSegmentsBySamples", "ps6000aQueryMaxSegmentsBySamples", c_uint32, [c_int16, c_uint64, c_int32, c_void_p, c_uint32], doc)

doc = """ PICO_STATUS ps6000aSetChannelOn
    (
        int16_t    handle,
        PICO_CHANNEL    channel,
        PICO_COUPLING    coupling,
        PICO_CONNECT_PROBE_RANGE    range,
        double    analogueOffset,
        PICO_BANDWIDTH_LIMITER    bandwidth
    ); """
ps6000a.make_symbol("_SetChannelOn", "ps6000aSetChannelOn", c_uint32, [c_int16, c_uint32, c_uint32, c_uint32, c_double, c_uint32], doc)

doc = """ PICO_STATUS ps6000aSetChannelOff
    (
        int16_t    handle,
        PICO_CHANNEL    channel
    ); """
ps6000a.make_symbol("_SetChannelOff", "ps6000aSetChannelOff", c_uint32, [c_int16, c_uint32], doc)

doc = """ PICO_STATUS ps6000aSetDigitalPortOn
    (
        int16_t    handle,
        PICO_CHANNEL    port,
        int16_t    *logicThresholdLevel,
        int16_t    logicThresholdLevelLength
        PICO_DIGITAL_PORT_HYSTERESIS    hysteresis
    ); """
ps6000a.make_symbol("_SetDigitalPortOn", "ps6000aSetDigitalPortOn", c_uint32, [c_int16, c_uint32, c_void_p, c_int16, c_uint32], doc)

doc = """ PICO_STATUS ps6000aSetDigitalPortOff
    (
        int16_t    handle,
        PICO_CHANNEL    port
    ); """
ps6000a.make_symbol("_SetDigitalPortOff", "ps6000aSetDigitalPortOff", c_uint32, [c_int16, c_uint32], doc)

doc = """ PICO_STATUS ps6000aGetTimebase
    (
        int16_t    handle,
        uint32_t    timebase,
        uint64_t    noSamples,
        double    *timeIntervalNanoseconds,
        uint64_t    *maxSamples,
        uint64_t    segmentIndex
    ); """
ps6000a.make_symbol("_GetTimebase", "ps6000aGetTimebase", c_uint32, [c_int16, c_uint32, c_uint64, c_void_p, c_void_p, c_uint64], doc)

doc = """ PICO_STATUS ps6000aSigGenWaveform
    (
        int16_t    handle,
        PICO_WAVE_TYPE    wavetype,
        int16_t    *buffer,
        uint16    bufferLength
    ); """
ps6000a.make_symbol("_SigGenWaveform", "ps6000aSigGenWaveform", c_uint32, [c_int16, c_uint32, c_void_p, c_uint16], doc)

doc = """ PICO_STATUS ps6000aSignGenRange
    (
        int16_t    handle, 
        double    peakToPeakVolts,
        double    offsetVolts
    ); """
ps6000a.make_symbol("_SigGenRange", "ps6000aSigGenRange", c_uint32, [c_int16, c_double, c_double], doc)

doc = """ PICO_STATUS ps6000aSigGenWaveformDutyCycle
    (
        int16_t    handle,
        double    dutyCyclePercent
    ); """
ps6000a.make_symbol("_SigGenWaveformDutyCycle", "ps6000aSigGenWaveformDutyCycle", c_uint32, [c_int16, c_double], doc)

doc = """ PICO_STATUS ps6000aSigGenTrigger
    (
        int16_t    handle,
        PICO_SIGGEN_TRIG_TYPE    triggerType,
        PICO_SIGGEN_TRIG_SOURCE    triggerSource,
        uint64_t    cycles,
        uint64_t    autoTriggerPicoSeconds
    ); """
ps6000a.make_symbol("_SigGenTrigger", "ps6000aSigGenTrigger", c_uint32, [c_int16, c_uint32, c_uint32, c_uint64, c_uint64], doc)

doc = """ PICO_STATUS ps6000aSigGenFilter
    (
        int16_t    handle,
        PICO_SIGGEN_FILTER_STATE    filterState
    ); """
ps6000a.make_symbol("_SigGenFilter", "ps6000aSigGenFilter", c_uint32, [c_int16, c_uint32], doc)

doc = """ PICO_STATUS ps6000aSigGenFrequency
    (
        int16_t    handle,
        double    frequencyHz
    ); """
ps6000a.make_symbol("_SigGenFrequency", "ps6000aSigGenFrequency", c_uint32, [c_int16, c_double], doc)

doc = """ PICO_STATUS ps6000aSigGenFrequencySweep
    (
        int16_t    handle,
        double    stopFrequencyHz,
        double    frequencyIncrement,
        double    dwellTimeSeconds,
        PICO_SWEEP_TYPE    sweepType
    ); """
ps6000a.make_symbol("_SigGenFrequencySweep", "ps6000aSigGenFrequencySweep", c_uint32, [c_int16, c_double, c_double, c_double, c_uint32], doc)

doc = """ PICO_STATUS ps6000aSigGenPhase
    (
        int16_t    handle,
        uint64_t    deltaPhase
    ); """
ps6000a.make_symbol("_SigGenPhase", "ps6000aSigGenPhase", c_uint32, [c_int16, c_uint64], doc)

doc = """ PICO_STATUS ps6000aSigGenPhaseSweep
    (
        int16_t    handle,
        uint64_t    stopDeltaPhase,
        uint64_t    deltaPhaseIncrement,
        uint64_t    dwellCount,
        PICO_SWEEP_TYPE    sweepType
    ); """
ps6000a.make_symbol("_SigGenPhaseSweep", "ps6000aSigGenPhaseSweep", c_uint32, [c_int16, c_uint64, c_uint64, c_uint64, c_uint32], doc)

doc = """ PICO_STATUS ps6000aSigGenClockManual
    (
        int16_t    handle,
        double    dacClockFrequency,
        uint64_t    prescaleRatio
    ); """
ps6000a.make_symbol("_SigGenClockManual", "ps6000aSigGenClockManual", c_uint32, [c_int16, c_double, c_uint64], doc)

doc = """ PICO_STATUS ps6000aSigGenSoftwareTriggerControl
    (
        int16_t    handle,
        PICO_SIGGEN_TRIG_TYPE    triggerState
    ); """
ps6000a.make_symbol("_SigGenSoftwareTriggerControl", "ps6000aSigGenSoftwareTriggerControl", c_uint32, [c_int16, c_uint32], doc)

doc = """ PICO_STATUS ps6000aSigGenApply
    (
        int16_t    handle,
        int16_t    sigGenEnabled,
        int16_t    sweepEnabled,
        int16_t    triggerEnabled,
        int16_t    automaticClockOptimisationEnabled,
        int16_t    overrideAutomaticClockAndPrescale,
        double    *frequency,
        double    *stopFrequency,
        double    *frequencyIncrement,
        double    *dwellTime
    ); """
ps6000a.make_symbol("_SigGenApply", "ps6000aSigGenApply", c_uint32, [c_int16, c_int16, c_int16, c_int16, c_int16, c_int16, c_void_p, c_void_p, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps6000aSigGenLimits
    (
        int16_t    handle,
        PICO_SIGGEN_PARAMETER    parameter,
        double    *minimumPermissibleValue,
        double    *maximumPermissibleValue,
        double    *step
    ); """
ps6000a.make_symbol("_SigGenLimits", "ps6000aSigGenLimits", c_uint32, [c_int16, c_uint32, c_double, c_double, c_double], doc)

doc = """ PICO_STATUS ps6000aSigGenFrequencyLimits
    (
        int16_t    handle,
        PICO_WAVE_TYPE    waveType,
        uint64_t    *numSamples,
        double    *startFrequency,
        int16_t    sweepEnabled,
        double    *manualDacClockFrequency,
        uint64_t    *manualPrescaleRatio,
        double    *maxStopFrequencyOut,
        double    *minFrequencyStepOut,
        double    *maxFrequencyStepOut,
        double    *minDwellTimeOut,
        double    *maxDwellTimeOut
    ); """
ps6000a.make_symbol("_SigGenFrequencyLimits", "ps6000aSigGenFrequencyLimits", c_uint32, [c_int16, c_uint32, c_void_p, c_void_p, c_int16, c_void_p, c_void_p, c_void_p, c_void_p, c_void_p, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps6000aSigGenPause
    (
        int16_t    handle
    ); """
ps6000a.make_symbol("_SigGenPause", "ps6000aSigGenPause", c_uint32, [c_int16], doc)

doc = """ PICO_STATUS ps6000aSigGenRestart
    (
        int16_t    handle
    ); """
ps6000a.make_symbol("_SigGenRestart", "ps6000aSigGenRestart", c_uint32, [c_int16], doc)

doc = """ PICO_STATUS ps6000aSetSimpleTrigger
    (
        int16_t    handle
        int16_t    enable,
        PICO_CHANNEL    source,
        int16_t    threshold,
        PICO_THRESHOLD_DIRECTION    direction,
        uint64_t    delay,
        uint32_t    autoTriggerMicroSeconds
    ); """
ps6000a.make_symbol("_SetSimpleTrigger", "ps6000aSetSimpleTrigger", c_uint32, [c_int16, c_int16, c_uint32, c_int16, c_uint32, c_uint64, c_uint32], doc)

doc = """ PICO_STATUS ps6000aTriggerWithinPreTriggerSamples
    (
        int16_t    handle,
        PICO_TRIGGER_WITHIN_PRE_TRIGGER    state
    ); """
ps6000a.make_symbol("_TriggerWithinPreTriggerSamples", "ps6000aTriggerWithinPreTriggerSamples", c_uint32, [c_int16, c_uint32], doc)

doc = """ PICO_STATUS ps6000aSetTriggerChannelProperties
    (
        int16_t    handle,
        PICO_TRIGGER_CHANNEL_PROPERTIES    *channelProperties,
        int16_t    nChannelProperties,
        int16_t    auxOutputEnable,
        uint32_t    autoTriggerMicroSeconds
    ); """
ps6000a.make_symbol("_SetTriggerChannelProperties", "ps6000aSetTriggerChannelProperties", c_uint32, [c_int16, c_void_p, c_int16, c_int16, c_uint32], doc)

doc = """ PICO_STATUS ps6000aSetTriggerChannelConditions
    (
        int16_t    handle,
        PICO_CONDITION    *conditions,
        int16_t    nConditions,
        PICO_ACTION    action
    ); """
ps6000a.make_symbol("_SetTriggerChannelConditions", "ps6000aSetTriggerChannelConditions", c_uint32, [c_int16, c_void_p, c_int16, c_uint32], doc)

doc = """ PICO_STATUS ps6000aSetTriggerChannelDirections
    (
        int16_t    handle,
        PICO_DIRECTION    *directions,
        int16_t    nDirections
    ); """
ps6000a.make_symbol("_SetTriggerChannelDirections", "ps6000aSetTriggerChannelDirections", c_uint32, [c_int16, c_void_p, c_int16], doc)

doc = """ PICO_STATUS ps6000aSetTriggerDelay
    (
        int16_t    handle,
        uint64_t    delay
    ); """
ps6000a.make_symbol("_SetTriggerDelay", "ps6000aSetTriggerDelay", c_uint32, [c_int16, c_uint64], doc)

doc = """ PICO_STATUS ps6000aSetPulseWidthQualifierProperties
    (
        int16_t    handle,
        uint32_t    lower,
        uint32_t    upper,
        PICO_PULSE_WIDTH_TYPE    type
    ); """
ps6000a.make_symbol("_SetPulseWidthQualifierProperties", "ps6000aSetPulseWidthQualifierProperties", c_uint32, [c_int16, c_uint32, c_uint32, c_uint32], doc)

doc = """ PICO_STATUS ps6000aSetPulseWidthQualifierConditions
    (
        int16_t    handle,
        PICO_CONDITION    *conditions,
        int16_t    nConditions,
        PICO_ACTION    action
    ); """
ps6000a.make_symbol("_SetPulseWidthQualifierConditions", "ps6000aSetPulseWidthQualifierConditions", c_uint32, [c_int16, c_void_p, c_int16, c_uint32], doc)

doc = """ PICO_STATUS ps6000aSetPulseWidthQualifierDirections
    (
        int16_t    handle,
        PICO_DIRECTION    *directions,
        int16_t    nDirections
    ); """
ps6000a.make_symbol("_SetPulseWidthQualifierDirections", "ps6000aSetPulseWidthQualifierDirections", c_uint32, [c_int16, c_void_p, c_int16], doc)

doc = """ PICO_STATUS ps6000aSetTriggerDigitalPortProperties
    (
        int16_t    handle,
        PICO_CHANNEL    port,
        PICO_DIGITAL_CHANNEL_DIRECTIONS    *directions,
        int16_t    nDirections
    ); """
ps6000a.make_symbol("_SetTriggerDigitalPortProperties", "ps6000aSetTriggerDigitalPortProperties", c_uint32, [c_int16, c_uint32, c_void_p, c_int16], doc)

doc = """ PICO_STATUS ps6000aSetPulseWidthDigitalPortProperties
    (
        int16_t    handle,
        PICO_CHANNEL    port,
        PICO_DIGITAL_CHANNEL_DIRECTIONS    *directions,
        int16_t    nDirections
    ); """
ps6000a.make_symbol("_SetPulseWidthDigitalPortProperties", "ps6000aSetPulseWidthDigitalPortProperties", c_uint32, [c_int16, c_uint32, c_void_p, c_int16], doc)

doc = """ PICO_STATUS ps6000aGetTriggerTimeOffset
    (
        int16_t    handle,
        int64_t    *time,
        PICO_TIME_UNITS    *timeUnits,
        uint64_t    segmentIndex
    ); """
ps6000a.make_symbol("_GetTriggerTimeOffset", "ps6000aGetTriggerTimeOffset", c_uint32, [c_int16, c_void_p, c_void_p, c_uint64], doc)

doc = """ PICO_STATUS ps6000aGetValuesTriggerTimeOffsetBulk
    (
        int16_t    handle,
        int64_t    *time,
        PICO_TIME_UNITS    *timeUnits,
        uint64_t    fromSegementIndex,
        uint64_t    toSegmentIndex
    ); """
ps6000a.make_symbol("_GetValuesTriggerTimeOffsetBulk", "ps6000aGetValuesTriggerTimeOffsetBulk", c_uint32, [c_int16, c_void_p, c_void_p, c_uint64, c_uint64], doc)

doc = """ PICO_STATUS ps6000aSetDataBuffer
    (
        int16_t    handle,
        PICO_CHANNEL    channel,
        PICO_POINTER    buffer,
        int32_t    nSamples,
        PICO_DATA_TYPE    dataType,
        uint64_t    waveform,
        PICO_RATIO_MODE    downSampleRatioMode,
        PICO_ACTION    action
    ); """
ps6000a.make_symbol("_SetDataBuffer", "ps6000aSetDataBuffer", c_uint32, [c_int16, c_uint32, c_void_p, c_int32, c_uint32, c_uint64, c_uint32, c_uint32], doc)

doc = """ PICO_STATUS ps6000aSetDataBuffers
    (
        int16_t    handle,
        PICO_CHANNEL    channel,
        PICO_POINTER    bufferMax,
        PICO_POINTER    bufferMin,
        int32_t    nSamples,
        PICO_DATA_TYPE    dataType,
        uint64_t    waveform,
        PICO_RATIO_MODE    downSampleRatioMode,
        PICO_ACTION    action
    ); """
ps6000a.make_symbol("_SetDataBuffers", "ps6000aSetDataBuffers", c_uint32, [c_int16, c_uint32, c_void_p, c_void_p, c_int32, c_uint32, c_uint64, c_uint32, c_uint32], doc)

doc = """ PICO_STATUS ps6000aRunBlock
    (
        int16_t    handle,
        uint64_t    noOfPreTriggerSamples,
        uint64_t    noOfPostTriggerSamples,
        uint32_t    timebase,
        double    *timeIndisposedMs,
        uint64_t    segmentIndex,
        ps6000aBlockReady    lpReady,
        PICO_POINTER    pParameter
    ); """
ps6000a.make_symbol("_RunBlock", "ps6000aRunBlock", c_uint32, [c_int16, c_uint64, c_uint64, c_uint32, c_void_p, c_uint64, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps6000aIsReady
    (
        int16_t    handle,
        int16_t    *ready
    ); """
ps6000a.make_symbol("_IsReady", "ps6000aIsReady", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps6000aRunStreaming
    (
        int16_t    handle,
        double    *sampleInterval,
        PICO_TIME_UNITS    sampleIntervalTimeUnits,
        uint64_t    maxPreTriggerSamples,
        uint64_t    maxPostTriggerSamples,
        int16_t    autoStop,
        uint64_t    downSampleRatio,
        PICO_RATIO_MODE    downSampelRatioMode
    ): """
ps6000a.make_symbol("_RunStreaming", "ps6000aRunStreaming", c_uint32, [c_int16, c_void_p, c_uint32, c_uint64, c_uint64, c_int16, c_uint64, c_uint32], doc)

doc = """ PICO_STATUS ps6000aGetStreamingLatestValues
    (
        int16_t    handle,
        PICO_STREAMING_DATA_INFO    *streamingDataInfo,
        uint64_t    nStreamingDataInfos,
        PICO_STREAMING_DATA_TRIGGER_INFO    *triggerInfo
    ); """
ps6000a.make_symbol("_GetStreamingLatestValues", "ps6000aGetStreamingLatestValues", c_uint32, [c_int16, c_void_p, c_uint64, c_void_p], doc)

doc = """ PICO_STATUS ps6000aNoOfStreamingValues
    (
        int16_t    handle,
        uint64_t    *noOfValues
    ); """
ps6000a.make_symbol("_NoOfStreamingValues", "ps6000aNoOfStreamingValues", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps6000aGetValues
    (
        int16_t    handle,
        uint64_t    startIndex,
        uint64_t    *noOfSamples,
        uint64_t    downSampleRatio,
        PICO_RATIO_MODE    downSampleRatioMode,
        uint64_t    segmentIndex,
        int16_t    *overflow
    ); """
ps6000a.make_symbol("_GetValues", "ps6000aGetValues", c_uint32, [c_int16, c_uint64, c_void_p, c_uint64, c_uint32, c_uint64, c_void_p], doc)

doc = """ PICO_STATUS ps6000aGetValuesBulk
    (
        int16_t    handle,
        uint64_t    startIndex,
        uint64_t    *noOfSamples,
        uint64_t    fromSegmentIndex,
        uint64_t    toSegmentIndex,
        uint64_t    downSampleRatio,
        PICO_RATIO_MODE    downSampleRatioMode,
        int16_t    *overflow
    ); """
ps6000a.make_symbol("_GetValuesBulk", "ps6000aGetValuesBulk", c_uint32, [c_int16, c_uint64, c_void_p, c_uint64, c_uint64, c_uint64, c_uint32, c_void_p], doc)

doc = """ PICO_STATUS ps6000aGetValuesAsync
    (
        int16_t    handle,
        uint64_t    startIndex,
        uint64_t    noOfSamples,
        uint64_t    downSampleRatio,
        PICO_RATIO_MODE    downSampleRatioMode,
        uint64_t    segmentIndex,
        PICO_POINTER    lpDataReady,
        PICO_POINTER    pParameter
    ); """
ps6000a.make_symbol("_GetValuesAsync", "ps6000aGetValuesAsync", c_uint32, [c_int16, c_uint64, c_uint64, c_uint64, c_uint32, c_uint64, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps6000aGetValuesBulkAsync
    (
        int16_t    handle,
        uint64_t    startIndex,
        uint64_t    noOfSamples,
        uint64_t    fromSegmentIndex,
        uint64_t    toSegmentIndex,
        uint64_t    downSampleRatio,
        PICO_RATIO_MODE    downSampleRatioMode,
        PICO_POINTER    lpDataReady,
        PICO_POINTER    pParameter
    ); """
ps6000a.make_symbol("_GetValuesBulkAsync", "ps6000aGetValuesBulkAsync", c_uint32, [c_int16, c_uint64, c_uint64, c_uint64, c_uint64, c_uint64, c_uint32, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps6000aGetValuesOverlapped
    (
        int16_t    handle,
        uint64_t    startIndex,
        uint64_t    *noOfSamples,
        uint64_t    downSampleRatio,
        PICO_RATIO_MODE    downSampleRatioMode,
        uint64_t    fromSegementIndex,
        uint64_t    toSegmentIndex,
        int16_t    *overflow
    ); """
ps6000a.make_symbol("_GetValuesOverlapped", "ps6000aGetValuesOverlapped", c_uint32, [c_int16, c_uint64, c_void_p, c_uint64, c_uint32, c_uint64, c_uint64, c_void_p], doc)

doc = """ PICO_STATUS ps6000aStopUsingGetValuesOverlapped
    (
        int16_t    handle
    ); """
ps6000a.make_symbol("_StopUsingGetValuesOverlapped", "ps6000aStopUsingGetValuesOverlapped", c_uint32, [c_int16], doc)

doc = """ PICO_STATUS ps6000aGetNoOfCaptures
    (
        int16_t    handle,
        uint64_t    *nCaptures
    ); """
ps6000a.make_symbol("_GetNoOfCaptures", "ps6000aGetNoOfCaptures", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps6000aGetNoOfProcessedCaptures
    (
        int16_t    handle,
        uint64_t    *nProcessedCaptures
    ); """
ps6000a.make_symbol("_GetNoOfProcessedCaptures", "ps6000aGetNoOfProcessedCaptures", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps6000aStop
    (
        int16_t    handle,
    ); """
ps6000a.make_symbol("_Stop", "ps6000aStop", c_uint32, [c_int16], doc)

doc = """ PICO_STATUS ps6000aSetNoOfCaptures
    (
        int16_t    handle,
        uint64_t    nCaptures
    ); """
ps6000a.make_symbol("_SetNoOfCaptures", "ps6000aSetNoOfCaptures", c_uint32, [c_int16, c_uint64], doc)

doc = """ PICO_STATUS ps6000aGetTriggerInfo
    (
        int16_t    handle,
        PICO_TRIGGER_INFO    *triggerInfo,
        uint64_t    firstSegmentIndex,
        uint64_t    segmentCount
    ); """
ps6000a.make_symbol("_getTriggerInfo", "ps6000aGetTriggerInfo", c_uint32, [c_int16, c_void_p, c_uint64, c_uint64], doc)

doc = """ PICO_STATUS ps6000aEnumerateUnits
    (
        int16_t    *count,
        int8_t    *serials,
        int16_t    *serialLth
    ); """
ps6000a.make_symbol("_EnumerateUnits", "ps6000aEnumerateUnits", c_uint32, [c_void_p, c_char_p, c_void_p], doc)

doc = """ PICO_STATUS ps6000aPingUnit
    (
        int16_t    handle
    ); """
ps6000a.make_symbol("_PingUnit", "ps6000aPingUnit", c_uint32, [c_int16], doc)

doc = """ PICO_STATUS ps6000aGetAnalogueOffsetLimits
    (
        int16_t    handle,
        PICO_CONNECT_PROBE_RANGE    range,
        PICO_COUPLING    coupling,
        double    *maximumVoltage,
        double    *minimumVoltage
    ); """
ps6000a.make_symbol("_GetAnalogueOffsetLimits", "ps6000aGetAnalogueOffsetLimits", c_uint32, [c_int16, c_uint32, c_uint32, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps6000aGetMinimumTimebaseStateless
    (
        int16_t    handle,
        PICO_CHANNEL_FLAGS    enabledChannelFlags,
        uint32_t    *timebase,
        double    *timeInterval,
        PICO_DEVICE_RESOLUTION    resolution
    ); """
ps6000a.make_symbol("_GetMinimumTimebaseStateless", "ps6000aGetMinimumTimebaseStateless", c_uint32, [c_int16, c_uint32, c_void_p, c_void_p, c_uint32], doc)

doc = """ PICO_STATUS ps6000aNearestSampleIntervalStateless
    (
        int16_t    handle,
        PICO_CHANNEL_FLAGS    enabledChannelFlags,
        double    timeIntervalRequested,
        PICO_DEVICE_RESOLUTION    resolution,
        uint32_t    *timebase,
        double    *timeIntervalAvailable
    ); """
ps6000a.make_symbol("_NearestSampleIntervalStateless", "ps6000aNearestSampleIntervalStateless", c_uint32, [c_int16, c_uint32, c_double, c_uint32, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps6000aChannelCombinationsStateless
    (
        int16_t    handle,
        PICO_CHANNEL_FLAGS    *channelFlagsCombinations,
        uint32_t    *nChannelCombinations,
        PICO_DEVICE_RESOLUTION    resolution,
        uint32_t    timebase
    ); """
ps6000a.make_symbol("_ChannelCombinationsStateless", "ps6000aChannelCombinationsStateless", c_uint32, [c_int16, c_void_p, c_void_p, c_uint32, c_uint32], doc)

doc = """ PICO_STATUS ps6000aSetDeviceResolution
    (
        int16_t    handle,
        PICO_DEVICE_RESOLUTION    resolution
    ); """
ps6000a.make_symbol("_SetDeviceResolution", "ps6000aSetDeviceResolution", c_uint32, [c_int16, c_uint32], doc)

doc = """ PICO_STATUS ps6000aGetDeviceResolution
    (
        int16_t    handle,
        PICO_DEVICE_RESOLUTION    *resolution
    ); """
ps6000a.make_symbol("_GetDeviceResolution", "ps6000aGetDeviceResolution", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps6000aQueryOutputEdgeDetect
    (
        int16_t    handle,
        int16_t    *state
    ); """
ps6000a.make_symbol("_QueryOutputEdgeDetect", "ps6000aQueryOutputEdgeDetect", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps6000aSetOutputEdgeDetect
    (
        int16_t    handle,
        int16_t    state
    ); """
ps6000a.make_symbol("_SetOutputEdgeDetect", "ps6000aSetOutputEdgeDetect", c_uint32, [c_int16, c_int16], doc)

doc = """ PICO_STATUS ps6000aGetScalingValues
    (
        int16_t    handle,
        PICO_SCALING_FACTORS_VALUES    *scalingValues,
        int16_t    nChannels
    ); """
ps6000a.make_symbol("_GetScalingValues", "ps6000aGetScalingValues", c_uint32, [c_int16, c_void_p, c_int16], doc)

doc = """ PICO_STATUS ps6000aGetAdcLimits
    (
        int16_t    handle,
        PICO_DEVICE_RESOLUTION    resolution,
        int16_t    *minValue,
        int16_t    *maxValue
    ); """
ps6000a.make_symbol("_GetAdcLimits", "ps6000aGetAdcLimits", c_uint32, [c_int16, c_uint32, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps6000aCheckForUpdate
    (
        int16_t    handle,
        PICO_VERSION    *current,
        PICO_VERSION    *update,
        uint16_t    *updateRequired
    ); """
ps6000a.make_symbol("_CheckForUpdate", "ps6000aCheckForUpdate", c_uint32, [c_int16, c_void_p, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps6000aStartFirmwareUpdate
    (
        int16_t    handle,
        PicoUpdateFirmwareProgress    progress 
    ); """
ps6000a.make_symbol("_StartFirmwareUpdate", "ps6000aStartFirmwareUpdate", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps6000aSetProbeInteractionCallback
    (
        int16_t    handle,
        PicoProbeInteractions    callback
    ); """
ps6000a.make_symbol("_SetProbeInteractionCallback", "ps6000aSetProbeInteractionCallback", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps6000aSetExternalReferenceInteractionCallback
    (
        int16_t    handle,
        PicoExternalReferenceInteractions    callback
    ); """
ps6000a.make_symbol("_SetExternalReferenceInteractionCallback", "ps6000aSetExternalReferenceInteractionCallback", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps6000aSetAWGOverrangeInteractionCallback
    (
        int16_t    handle,
        PicoAWGOverrangeInteractions    callback
    ); """
ps6000a.make_symbol("_SetAWGOVerrangeInteractionCallback", "ps6000aSetAWGOverrangeInteractionCallback", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps6000aSetTemperatureSensorInteractioNCallback
    (
        int16_t    handle,
        PicoTemperatureSensorInteractions    callback
    ); """
ps6000a.make_symbol("_SetTemperatureSensroInteractionCallback", "ps6000aSetTemperatureSensorInteractionCallback", c_uint32, [c_int16, c_void_p], doc)


import asyncio
import concurrent.futures
import functools

from ctypes import POINTER, c_int16, c_uint32

from picosdk.ps2000 import ps2000
from picosdk.functions import assert_pico2000_ok
from picosdk.ctypes_wrapper import C_CALLBACK_FUNCTION_FACTORY

from enum import IntEnum


class Channel(IntEnum):
    PS2000_CHANNEL_A = 0
    PS2000_CHANNEL_B = 1


class PotentialRange(IntEnum):
    PS2000_20MV = 1
    PS2000_50MV = 2
    PS2000_100MV = 3
    PS2000_200MV = 4
    PS2000_500MV = 5
    PS2000_1V = 6
    PS2000_2V = 7
    PS2000_5V = 8
    PS2000_10V = 9
    PS2000_20V = 10


CALLBACK = C_CALLBACK_FUNCTION_FACTORY(None, POINTER(POINTER(c_int16)), c_int16, c_uint32, c_int16, c_int16, c_uint32)


# reimplement this because the other one only takes ctypes
def adc_to_mv(values, range_, bitness=16):
    v_ranges = [10, 20, 50, 100, 200, 500, 1_000, 2_000, 5_000, 10_000, 20_000]

    return [(x * v_ranges[range_]) / (2**(bitness - 1) - 1) for x in values]


class StreamingDevice:
    def __init__(self, potential_range=PotentialRange.PS2000_50MV):
        self.device = ps2000.open_unit()
        self.potential_range = potential_range
        # create an event loop
        self.loop = asyncio.new_event_loop()
        # create a thread-pool executor
        self.executor = concurrent.futures.ThreadPoolExecutor()

        self.waiting_blocks = {}

        res = ps2000.ps2000_set_channel(self.device.handle, Channel.PS2000_CHANNEL_A, True, True, potential_range)
        assert_pico2000_ok(res)

        # start 'fast-streaming' mode
        res = ps2000.ps2000_run_streaming_ns(
            self.device.handle,
            100,
            3,
            30_000,
            False,
            1,
            15_000
        )
        assert_pico2000_ok(res)

    async def process_value(self, mv_value):
        # overload
        pass

    def get_streamed_values(self, block_id):
        def get_overview_buffers(buffers, _overflow, _triggered_at, _triggered, _auto_stop, n_values):
            adc_values = buffers[0][0:n_values]

            mv_values = adc_to_mv(adc_values, self.potential_range)

            self.waiting_blocks[block_id] = mv_values

        callback = CALLBACK(get_overview_buffers)

        res = ps2000.ps2000_get_streaming_last_values(
            self.device.handle,
            callback
        )
        assert_pico2000_ok(res)

    def start(self):
        async def gather_values():
            block_id = 0
            while True:
                await asyncio.sleep(0.5)

                await self.loop.run_in_executor(self.executor, functools.partial(self.get_streamed_values, block_id))

                block_id += 1

        async def poll_waiting():
            next_block = 0
            while True:
                if block := self.waiting_blocks.get(next_block):
                    next_block += 1
                    for n in block:
                        await self.process_value(n)

                await asyncio.sleep(0.25)

        try:
            self.loop.create_task(gather_values())
            self.loop.run_until_complete(poll_waiting())
        finally:
            self.loop.close()
            ps2000.ps2000_stop(self.device.handle)
            ps2000.ps2000_close(self.device.handle)


stream = StreamingDevice()
stream.start()



#
# Copyright (C) 2018 Pico Technology Ltd. See LICENSE file for terms.
#
"""
Unit tests for the wrapper functions for discovering devices
"""

from __future__ import print_function

from test.test_helpers import DriverTest, drivers_with_device_connected
from picosdk.device import Device
from picosdk.errors import DeviceNotFoundError
import picosdk.discover as dut


class OpenCloseTest(DriverTest):
    def test_find_unit_success(self):
        """test_find_unit_success
        note: test assumes you have set test_helpers.drivers_with_device_connected"""
        if not drivers_with_device_connected:
            return

        device = dut.find_unit()
        try:
            self.assertIsInstance(device, Device)
            self.assertIn(device.driver.name, drivers_with_device_connected)
        finally:
            device.close()

    def test_find_all_units_success(self):
        """test_find_all_units_success
        note: test assumes you have set test_helpers.drivers_with_device_connected"""
        if not drivers_with_device_connected:
            return

        devices = dut.find_all_units()

        try:
            for device in devices:
                self.assertIsInstance(device, Device)
                self.assertIn(device.driver.name, drivers_with_device_connected)
            self.assertEqual(set(device.driver.name for device in devices), set(drivers_with_device_connected))
        finally:
            for device in devices:
                device.close()

    def test_find_unit_failure(self):
        devices = []
        threw = False
        try:
            while len(devices) <= len(drivers_with_device_connected):
                devices.append(dut.find_unit())
        except DeviceNotFoundError:
            threw = True
        finally:
            for device in devices:
                device.close()
        self.assertEqual(len(drivers_with_device_connected), len(devices))
        self.assertTrue(threw)

    def test_find_all_units_failure(self):
        devices = []
        threw = False
        try:
            devices = dut.find_all_units()
            dut.find_all_units()
        except DeviceNotFoundError:
            threw = True
        finally:
            for device in devices:
                device.close()
        self.assertTrue(threw)



#
# Copyright (C) 2020 Pico Technology Ltd. See LICENSE file for terms.
#
# PS6000 A STREAMING MODE EXAMPLE
# This example opens a 6000a driver device, sets up one channel then collects a streamed set of data.
# This data is then plotted as mV against time in ns.

import ctypes
# import numpy as np
from picosdk.ps6000a import ps6000a as ps
from picosdk.PicoDeviceEnums import picoEnum as enums
from picosdk.PicoDeviceStructs import picoStruct as structs
import matplotlib.pyplot as plt
from picosdk.functions import adc2mV, assert_pico_ok
from picosdk.constants import PICO_STATUS
import time

# Create chandle and status ready for use
chandle = ctypes.c_int16()
status = {}

# Open 6000 A series PicoScope
# returns handle to chandle for use in future API functions
resolution = enums.PICO_DEVICE_RESOLUTION["PICO_DR_8BIT"]
status["openunit"] = ps.ps6000aOpenUnit(ctypes.byref(chandle), None, resolution)
assert_pico_ok(status["openunit"])

# Set channel A on
# handle = chandle
channelA = enums.PICO_CHANNEL["PICO_CHANNEL_A"]
coupling = enums.PICO_COUPLING["PICO_DC"]
channelRange = 7
# analogueOffset = 0 V
bandwidth = enums.PICO_BANDWIDTH_LIMITER["PICO_BW_FULL"]
status["setChannelA"] = ps.ps6000aSetChannelOn(chandle, channelA, coupling, channelRange, 0, bandwidth)
assert_pico_ok(status["setChannelA"])

# set channel B-H off
for x in range(1, 7, 1):
    channel = x
    status["setChannel", x] = ps.ps6000aSetChannelOff(chandle, channel)
    assert_pico_ok(status["setChannel", x])

# Set number of samples to be collected
noOfPreTriggerSamples = 100000
noOfPostTriggerSamples = 900000
nSamples = noOfPostTriggerSamples + noOfPreTriggerSamples

# Set simple trigger on channel A, 1 V rising with 1 s autotrigger
# handle = chandle
# enable = 1
source = channelA
# threshold = 1000 mV
direction = enums.PICO_THRESHOLD_DIRECTION["PICO_RISING"]
# delay = 0 s
# autoTriggerMicroSeconds = 1000000 us
status["setSimpleTrigger"] = ps.ps6000aSetSimpleTrigger(chandle, 1, source, 1000, direction, 0, 1000000)
assert_pico_ok(status["setSimpleTrigger"])

# create buffers
maxBuffers = 10

bufferA = ((ctypes.c_int16 * nSamples) * 10)()

print(bufferA)
# Set data buffers
# handle = chandle
# channel = channelA
# bufferMax = bufferAMax
# bufferMin = bufferAMin
# nSamples = nSamples
dataType = enums.PICO_DATA_TYPE["PICO_INT16_T"]
waveform = 0
downSampleMode = enums.PICO_RATIO_MODE["PICO_RATIO_MODE_RAW"]
clear = enums.PICO_ACTION["PICO_CLEAR_ALL"]
add = enums.PICO_ACTION["PICO_ADD"]
action = clear | add  # PICO_ACTION["PICO_CLEAR_WAVEFORM_CLEAR_ALL"] | PICO_ACTION["PICO_ADD"]
status["setDataBuffers"] = ps.ps6000aSetDataBuffer(chandle, channelA, ctypes.byref(bufferA[0]), nSamples, dataType,
                                                   waveform, downSampleMode, action)
assert_pico_ok(status["setDataBuffers"])

# Run streaming
sampleInterval = ctypes.c_double(1)
sampleIntervalTimeUnits = enums.PICO_TIME_UNITS["PICO_US"]
autoStop = 0
downSampleRatio = 1

status["runStreaming"] = ps.ps6000aRunStreaming(chandle, ctypes.byref(sampleInterval), sampleIntervalTimeUnits,
                                                noOfPreTriggerSamples, noOfPostTriggerSamples, autoStop,
                                                downSampleRatio, downSampleMode)
assert_pico_ok(status["runStreaming"])

streamData = structs.PICO_STREAMING_DATA_INFO(channelA, downSampleRatio, dataType, 0, 0, 0, 0)

streamTrigger = structs.PICO_STREAMING_DATA_TRIGGER_INFO(0, 0, 0)

count = 1

actionB = add

picoOk = PICO_STATUS["PICO_OK"]

while count <= maxBuffers:

    status["getStreamingLatestValues"] = ps.ps6000aGetStreamingLatestValues(chandle, ctypes.byref(streamData), 1,
                                                                            ctypes.byref(streamTrigger))

    if status["getStreamingLatestValues"] == picoOk:
        # do nothing
        time.sleep(0.01)
    else:
        count = count + 1
        if count <= maxBuffers:
            status["setDataBuffer"] = ps.ps6000aSetDataBuffer(chandle, channelA, ctypes.byref(bufferA[count - 1]),
                                                              nSamples, dataType, waveform, downSampleMode, actionB)
            assert_pico_ok(status["setDataBuffer"])
            print(count)

print("streaming finished")

# get max ADC value
# handle = chandle
minADC = ctypes.c_int16()
maxADC = ctypes.c_int16()
status["getAdcLimits"] = ps.ps6000aGetAdcLimits(chandle, resolution, ctypes.byref(minADC), ctypes.byref(maxADC))
assert_pico_ok(status["getAdcLimits"])

# convert ADC counts data to mV
bufferAmV = ((ctypes.c_int16 * nSamples) * 10)()
for j in range(0, 9):
    A = bufferA[j]
    bufferAmV[j] = adc2mV(A, channelRange, maxADC)

# Close the scope
status["closeunit"] = ps.ps6000aCloseUnit(chandle)
assert_pico_ok(status["closeunit"])

print(status)



#
# Copyright (C) 2015-2018 Pico Technology Ltd. See LICENSE file for terms.
#
"""
This is a Python module defining the functions from the ps4000aApi.h C header
file for PicoScope 4000 Series oscilloscopes using the ps4000a driver API
functions.
"""

from ctypes import *
from picosdk.library import Library
from picosdk.constants import make_enum
from picosdk.ctypes_wrapper import C_CALLBACK_FUNCTION_FACTORY

class Ps4000alib(Library):
    def __init__(self):
        super(Ps4000alib, self).__init__("ps4000a")


ps4000a = Ps4000alib()

ps4000a.PS4000A_COUPLING = make_enum([
    'PS4000A_AC',
    'PS4000A_DC',
])

# Just use AC and DC.
ps4000a.PICO_COUPLING = {k[-2:]: v for k, v in ps4000a.PS4000A_COUPLING.items()}

# A tuple in an enum like this is 2 names for the same value.
ps4000a.PS4000A_CHANNEL = {
    "PS4000A_CHANNEL_A" : 0,
    "PS4000A_CHANNEL_B" : 1,
    "PS4000A_CHANNEL_C" : 2,
    "PS4000A_CHANNEL_D" : 3,
    ("PS4000A_CHANNEL_E", "PS4000A_MAX_4_CHANNELS") : 4,
    "PS4000A_CHANNEL_F" : 5,
    "PS4000A_CHANNEL_G" : 6 ,
    "PS4000A_CHANNEL_H" : 7,
    ("PS4000A_MAX_CHANNELS", "PS4000A_EXTERNAL") : 8,
    "PS4000A_TRIGGER_AUX" : 9,
    "PS4000A_MAX_TRIGGER_SOURCES" : 10,
    "PS4000A_PULSE_WIDTH_SOURCE" : 0x10000000
}

ps4000a.PICO_CHANNEL = {k[19:]: v for k, v in ps4000a.PS4000A_CHANNEL.items()}



# The voltage ranges for this driver are so oddly defined, that it is easier to describe them as a literal than trying
# to use make_enum:

def _define_ranges():
    # These are lower case to keep the python style police happy. They are available in the range dictionary with their
    # usual upper case names precisely as in the C header file.
    pico_x1_probe_10mv = 0
    pico_x1_probe_20mv = 1
    pico_x1_probe_50mv = 2
    pico_x1_probe_100mv = 3
    pico_x1_probe_200mv = 4
    pico_x1_probe_500mv = 5
    pico_x1_probe_1v = 6
    pico_x1_probe_2v = 7
    pico_x1_probe_5v = 8
    pico_x1_probe_10v = 9
    pico_x1_probe_20v = 10
    pico_x1_probe_50v = 11
    pico_x1_probe_100v = 12
    pico_x1_probe_200v = 13
    pico_x1_probe_ranges = (pico_x1_probe_200v + 1) - pico_x1_probe_10mv

    pico_ps4000a_resistance_315k = 0x00000200
    pico_ps4000a_resistance_1100k = pico_ps4000a_resistance_315k + 1
    pico_ps4000a_resistance_10m = pico_ps4000a_resistance_315k + 2
    pico_ps4000a_max_resistance_ranges = (pico_ps4000a_resistance_10m + 1) - pico_ps4000a_resistance_315k
    pico_ps4000a_resistance_adcv_flag = 0x10000000

    pico_connect_probe_off = 1024

    pico_d9_bnc_10mv = 0
    pico_d9_bnc_20mv = 1
    pico_d9_bnc_50mv = 2
    pico_d9_bnc_100mv = 3
    pico_d9_bnc_200mv = 4
    pico_d9_bnc_500mv = 5
    pico_d9_bnc_1v = 6
    pico_d9_bnc_2v = 7
    pico_d9_bnc_5v = 8
    pico_d9_bnc_10v = 9
    pico_d9_bnc_20v = 10
    pico_d9_bnc_50v = 11
    pico_d9_bnc_100v = 12
    pico_d9_bnc_200v = 13
    pico_max_d9_bnc_ranges = (pico_d9_bnc_200v + 1) - pico_d9_bnc_10mv

    pico_d9_2x_bnc_10mv = pico_d9_bnc_10mv
    pico_d9_2x_bnc_20mv = pico_d9_bnc_20mv
    pico_d9_2x_bnc_50mv = pico_d9_bnc_50mv
    pico_d9_2x_bnc_100mv = pico_d9_bnc_100mv
    pico_d9_2x_bnc_200mv = pico_d9_bnc_200mv
    pico_d9_2x_bnc_500mv = pico_d9_bnc_500mv
    pico_d9_2x_bnc_1v = pico_d9_bnc_1v
    pico_d9_2x_bnc_2v = pico_d9_bnc_2v
    pico_d9_2x_bnc_5v = pico_d9_bnc_5v
    pico_d9_2x_bnc_10v = pico_d9_bnc_10v
    pico_d9_2x_bnc_20v = pico_d9_bnc_20v
    pico_d9_2x_bnc_50v = pico_d9_bnc_50v
    pico_d9_2x_bnc_100v = pico_d9_bnc_100v
    pico_d9_2x_bnc_200v = pico_d9_bnc_200v
    pico_max_d9_2x_bnc_ranges = (pico_d9_2x_bnc_200v + 1) - pico_d9_2x_bnc_10mv

    pico_differential_10mv = pico_d9_bnc_10mv
    pico_differential_20mv = pico_d9_bnc_20mv
    pico_differential_50mv = pico_d9_bnc_50mv
    pico_differential_100mv = pico_d9_bnc_100mv
    pico_differential_200mv = pico_d9_bnc_200mv
    pico_differential_500mv = pico_d9_bnc_500mv
    pico_differential_1v = pico_d9_bnc_1v
    pico_differential_2v = pico_d9_bnc_2v
    pico_differential_5v = pico_d9_bnc_5v
    pico_differential_10v = pico_d9_bnc_10v
    pico_differential_20v = pico_d9_bnc_20v
    pico_differential_50v = pico_d9_bnc_50v
    pico_differential_100v = pico_d9_bnc_100v
    pico_differential_200v = pico_d9_bnc_200v
    pico_max_differential_ranges = (pico_differential_200v + 1) - pico_differential_10mv,

    pico_current_clamp_200a_2ka_1a = 4000
    pico_current_clamp_200a_2ka_2a = pico_current_clamp_200a_2ka_1a + 1
    pico_current_clamp_200a_2ka_5a = pico_current_clamp_200a_2ka_1a + 2
    pico_current_clamp_200a_2ka_10a = pico_current_clamp_200a_2ka_1a + 3
    pico_current_clamp_200a_2ka_20a = pico_current_clamp_200a_2ka_1a + 4
    pico_current_clamp_200a_2ka_50a = pico_current_clamp_200a_2ka_1a + 5
    pico_current_clamp_200a_2ka_100a = pico_current_clamp_200a_2ka_1a + 6
    pico_current_clamp_200a_2ka_200a = pico_current_clamp_200a_2ka_1a + 7
    pico_current_clamp_200a_2ka_500a = pico_current_clamp_200a_2ka_1a + 8
    pico_current_clamp_200a_2ka_1000a = pico_current_clamp_200a_2ka_1a + 9
    pico_current_clamp_200a_2ka_2000a = pico_current_clamp_200a_2ka_1a + 10
    pico_max_current_clamp_200a_2ka_ranges = (pico_current_clamp_200a_2ka_2000a + 1) - pico_current_clamp_200a_2ka_1a

    pico_current_clamp_40a_100ma = 5000
    pico_current_clamp_40a_200ma = pico_current_clamp_40a_100ma + 1
    pico_current_clamp_40a_500ma = pico_current_clamp_40a_100ma + 2
    pico_current_clamp_40a_1a = pico_current_clamp_40a_100ma + 3
    pico_current_clamp_40a_2a = pico_current_clamp_40a_100ma + 4
    pico_current_clamp_40a_5a = pico_current_clamp_40a_100ma + 5
    pico_current_clamp_40a_10a = pico_current_clamp_40a_100ma + 6
    pico_current_clamp_40a_20a = pico_current_clamp_40a_100ma + 7
    pico_current_clamp_40a_40a = pico_current_clamp_40a_100ma + 8
    pico_max_current_clamp_40a_ranges = (pico_current_clamp_40a_40a + 1) - pico_current_clamp_40a_100ma

    pico_1kv_2_5v = 6003
    pico_1kv_5v = pico_1kv_2_5v + 1
    pico_1kv_12_5v = pico_1kv_2_5v + 2
    pico_1kv_25v = pico_1kv_2_5v + 3
    pico_1kv_50v = pico_1kv_2_5v + 4
    pico_1kv_125v = pico_1kv_2_5v + 5
    pico_1kv_250v = pico_1kv_2_5v + 6
    pico_1kv_500v = pico_1kv_2_5v + 7
    pico_1kv_1000v = pico_1kv_2_5v + 8
    pico_max_1kv_ranges = (pico_1kv_1000v + 1) - pico_1kv_2_5v

    return {k.upper(): v for k, v in locals().items() if k.startswith("pico")}


ps4000a.PICO_CONNECT_PROBE_RANGE = _define_ranges()


def process_enum(enum):
    """The PS4000a range enum is complicated enough that we need some clearer logic:"""
    import re
    pattern = re.compile(r'PICO_X1_PROBE_([0-9]+M?)V')

    voltage_range = {}

    for enum_item_name, enum_item_value in enum.items():
        match = pattern.match(enum_item_name)
        if match is None:
            continue
        voltage_string = match.group(1)
        voltage = float(voltage_string) if voltage_string[-1] != 'M' else (0.001 * float(voltage_string[:-1]))

        voltage_range[enum_item_value] = voltage

    return voltage_range


ps4000a.PICO_VOLTAGE_RANGE = process_enum(ps4000a.PICO_CONNECT_PROBE_RANGE)

ps4000a.PS4000A_RATIO_MODE = {
    'PS4000A_RATIO_MODE_NONE': 0,
    'PS4000A_RATIO_MODE_AGGREGATE': 1,
    'PS4000A_RATIO_MODE_DECIMATE': 2,
    'PS4000A_RATIO_MODE_AVERAGE': 4,
}

ps4000a.PICO_RATIO_MODE = {k[19:]: v for k, v in ps4000a.PS4000A_RATIO_MODE.items()}

ps4000a.PS4000A_TIME_UNITS = make_enum([
    'PS4000A_FS',
    'PS4000A_PS',
    'PS4000A_NS',
    'PS4000A_US',
    'PS4000A_MS',
    'PS4000A_S',
    'PS4000A_MAX_TIME_UNITS',
])

ps4000a.PS4000A_WAVE_TYPE = make_enum([
	'PS4000A_SINE',
	'PS4000A_SQUARE',
	'PS4000A_TRIANGLE',
	'PS4000A_RAMP_UP',
	'PS4000A_RAMP_DOWN',
	'PS4000A_SINC',
	'PS4000A_GAUSSIAN',
	'PS4000A_HALF_SINE',
	'PS4000A_DC_VOLTAGE',
	'PS4000A_WHITE_NOISE',
	'PS4000A_MAX_WAVE_TYPES',
])

ps4000a.PS4000A_SWEEP_TYPE = make_enum([
	'PS4000A_UP',
	'PS4000A_DOWN',
	'PS4000A_UPDOWN',
	'PS4000A_DOWNUP',
	'PS4000A_MAX_SWEEP_TYPES',
])

ps4000a.PS4000A_SIGGEN_TRIG_TYPE = make_enum([
	'PS4000A_SIGGEN_RISING',
	'PS4000A_SIGGEN_FALLING',
	'PS4000A_SIGGEN_GATE_HIGH',
	'PS4000A_SIGGEN_GATE_LOW',
])

ps4000a.PS4000A_SIGGEN_TRIG_SOURCE = make_enum([
	'PS4000A_SIGGEN_NONE',
	'PS4000A_SIGGEN_SCOPE_TRIG',
	'PS4000A_SIGGEN_AUX_IN',
	'PS4000A_SIGGEN_EXT_IN',
	'PS4000A_SIGGEN_SOFT_TRIG',
])

ps4000a.PS4000A_INDEX_MODE = make_enum([
	'PS4000A_SINGLE',
	'PS4000A_DUAL',
	'PS4000A_QUAD',
	'PS4000A_MAX_INDEX_MODES',
])

ps4000a.PS4000A_EXTRA_OPERATIONS = make_enum([
	'PS4000A_ES_OFF',
	'PS4000A_WHITENOISE',
	'PS4000A_PRBS',
])

ps4000a.PS4000A_CONDITIONS_INFO = make_enum([
    'PS4000A_CLEAR',
    'PS4000A_ADD',
])

ps4000a.PS4000A_THRESHOLD_DIRECTION = make_enum([
    ("PS4000A_ABOVE", "PS4000A_INSIDE"),
    ("PS4000A_BELOW", "PS4000A_OUTSIDE"),
    ("PS4000A_RISING", "PS4000A_ENTER", "PS4000A_NONE"),
    ("PS4000A_FALLING", "PS4000A_EXIT"),
    ("PS4000A_RISING_OR_FALLING", "PS4000A_ENTER_OR_EXIT"),
    "PS4000A_ABOVE_LOWER",
    "PS4000A_BELOW_LOWER",
    "PS4000A_RISING_LOWER",
    "PS4000A_FALLING_LOWER",
    "PS4000A_POSITIVE_RUNT",
    "PS4000A_NEGATIVE_RUNT",
])

ps4000a.PS4000A_THRESHOLD_MODE = make_enum([
    "PS4000A_LEVEL",
    "PS4000A_WINDOW"
])

ps4000a.PS4000A_TRIGGER_STATE = make_enum([
    "PS4000A_DONT_CARE",
    "PS4000A_TRUE",
    "PS4000A_FALSE"
])

ps4000a.PS4000A_PULSE_WIDTH_TYPE = make_enum([
    "PW_TYPE NONE",
    "PW_TYPE_LESS_THAN",
    "PW_TYPE_GREATER_THAN",
    "PW_TYPE_IN_RANGE",
    "PW_TYPE_OUT_OF_RANGE"
])

class PS4000A_CONDITION (Structure):
	_pack_ = 1
	_fields_ = [("source", c_int32),
				("condition", c_int16)]
                
ps4000a.PS4000A_CONDITION = PS4000A_CONDITION

class PS4000A_DIRECTION(Structure):
    _pack_ = 1
    _fields_ = [("channel", c_int32),
                ("direction", c_int32),
                ("mode", c_int32)]
                
ps4000a.PS4000A_DIRECTION = PS4000A_DIRECTION

class PS4000A_TRIGGER_CHANNEL_PROPERTIES(Structure):
    _pack_ = 1
    _fields_ = [("thresholdUpper", c_int16),
                ("thresholdUpperHysteresis", c_uint16),
                ("thresholdLower", c_int16),
                ("thresholdLowerHysteresis", c_uint16),
                ("channel", c_int32),
                ("thresholdMode", c_int32)]
                
ps4000a.PS4000A_TRIGGER_CHANNEL_PROPERTIES = PS4000A_TRIGGER_CHANNEL_PROPERTIES

doc = """ PICO_STATUS ps4000aOpenUnit
    (
        int16_t *handle,
        int8_t  *serial
    ); """
ps4000a.make_symbol("_OpenUnit", "ps4000aOpenUnit", c_uint32, [c_void_p, c_char_p], doc)

doc = """ PICO_STATUS ps4000aOpenUnitAsync
    (
        int16_t *status,
        int8_t  *serial
    ); """
ps4000a.make_symbol("_OpenUnitAsync", "ps4000aOpenUnitAsync", c_uint32, [c_void_p, c_char_p], doc)

doc = """ PICO_STATUS ps4000aOpenUnitProgress
    (
        int16_t *handle,
        int16_t *progressPercent,
        int16_t *complete
    ); """
ps4000a.make_symbol("_OpenUnitProgress", "ps4000aOpenUnitProgress", c_uint32, [c_void_p, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps4000aGetUnitInfo
    (
        int16_t    handle,
        int8_t    *string,
        int16_t    stringLength,
        int16_t   *requiredSize,
        PICO_INFO  info
    ); """
ps4000a.make_symbol("_GetUnitInfo", "ps4000aGetUnitInfo", c_uint32, [c_int16, c_char_p, c_int16, c_void_p, c_uint32],
                    doc)

doc = """ PICO_STATUS ps4000aFlashLed
    (
        int16_t  handle,
        int16_t  start
    ); """
ps4000a.make_symbol("_FlashLed", "ps4000aFlashLed", c_uint32, [c_int16, c_int16], doc)

doc = """ PICO_STATUS ps4000aSetChannelLed
    (
        int16_t                      handle,
        PS4000A_CHANNEL_LED_SETTING *ledStates,
        uint16_t                     nLedStates
    ); """
ps4000a.make_symbol("_SetChannelLed", "ps4000aSetChannelLed", c_uint32, [c_int16, c_void_p, c_uint16], doc)

doc = """ PICO_STATUS ps4000aIsLedFlashing
    (
        int16_t  handle,
        int16_t *status
    ); """
ps4000a.make_symbol("_IsLedFlashing", "ps4000aIsLedFlashing", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps4000aCloseUnit
    (
        int16_t  handle
    ); """
ps4000a.make_symbol("_CloseUnit", "ps4000aCloseUnit", c_uint32, [c_int16, ], doc)

doc = """ PICO_STATUS ps4000aMemorySegments
    (
        int16_t   handle,
        uint32_t  nSegments,
        int32_t  *nMaxSamples
    ); """
ps4000a.make_symbol("_MemorySegments", "ps4000aMemorySegments", c_uint32, [c_int16, c_uint32, c_void_p], doc)

doc = """ PICO_STATUS ps4000aSetChannel
    (
        int16_t           handle,
        PS4000A_CHANNEL   channel,
        int16_t           enabled,
        PS4000A_COUPLING  type,
        PS4000A_RANGE     range,
        float             analogOffset
    ); """
ps4000a.make_symbol("_SetChannel", "ps4000aSetChannel", c_uint32,
                    [c_int16, c_int32, c_int16, c_int32, c_int32, c_float], doc)

doc = """ PICO_STATUS ps4000aSetBandwidthFilter
    (
        int16_t                    handle,
        PS4000A_CHANNEL            channel,
        PS4000A_BANDWIDTH_LIMITER  bandwidth
    ); """
ps4000a.make_symbol("_SetBandwidthFilter", "ps4000aSetBandwidthFilter", c_uint32, [c_int16, c_int32, c_int32], doc)

doc = """ PICO_STATUS ps4000aApplyResistanceScaling
    (
        int16_t          handle,
        PS4000A_CHANNEL  channel,
        PS4000A_RANGE    range,
        int16_t         *bufferMax,
        int16_t         *bufferMin,
        uint32_t         buffertLth,
        int16_t         *overflow
    ); """
ps4000a.make_symbol("_ApplyResistanceScaling", "ps4000aApplyResistanceScaling", c_uint32,
                    [c_int16, c_int32, c_int32, c_int16, c_int16, c_uint32, c_int16], doc)

doc = """ PICO_STATUS ps4000aGetTimebase
    (
        int16_t   handle,
        uint32_t  timebase,
        int32_t   noSamples,
        int32_t  *timeIntervalNanoseconds,
        int32_t  *maxSamples,
        uint32_t  segmentIndex
    ); """
ps4000a.make_symbol('_GetTimebase', 'ps4000aGetTimebase', c_uint32,
                    [c_int16, c_uint32, c_int32, c_void_p, c_void_p, c_uint32], doc)

doc = """ PICO_STATUS ps4000aGetTimebase2
    (
        int16_t   handle,
        uint32_t  timebase,
        int32_t   noSamples,
        float    *timeIntervalNanoseconds,
        int32_t  *maxSamples,
        uint32_t  segmentIndex
    ); """
ps4000a.make_symbol("_GetTimebase2", "ps4000aGetTimebase2", c_uint32,
                    [c_int16, c_uint32, c_int32, c_void_p, c_void_p, c_uint32], doc)

doc = """ PICO_STATUS ps4000aSetSigGenArbitrary
    (
        int16_t                     handle,
        int32_t                     offsetVoltage,
        uint32_t                    pkToPk,
        uint32_t                    startDeltaPhase,
        uint32_t                    stopDeltaPhase,
        uint32_t                    deltaPhaseIncrement,
        uint32_t                    dwellCount,
        int16_t                    *arbitraryWaveform,
        int32_t                     arbitraryWaveformSize,
        PS4000A_SWEEP_TYPE          sweepType,
        PS4000A_EXTRA_OPERATIONS    operation,
        PS4000A_INDEX_MODE          indexMode,
        uint32_t                    shots,
        uint32_t                    sweeps,
        PS4000A_SIGGEN_TRIG_TYPE    triggerType,
        PS4000A_SIGGEN_TRIG_SOURCE  triggerSource,
        int16_t                     extInThreshold
    ); """
ps4000a.make_symbol("_SetSigGenArbitrary", "ps4000aSetSigGenArbitrary", c_uint32,
                    [c_int16, c_int32, c_uint32, c_uint32, c_uint32, c_uint32, c_uint32, c_void_p,
                     c_int32, c_int32, c_int32, c_int32, c_uint32, c_uint32, c_int32, c_int32, c_int16], doc)

doc = """ PICO_STATUS ps4000aSetSigGenBuiltIn
    (
        int16_t                     handle,
        int32_t                     offsetVoltage,
        uint32_t                    pkToPk,
        PS4000A_WAVE_TYPE           waveType,
        double                      startFrequency,
        double                      stopFrequency,
        double                      increment,
        double                      dwellTime,
        PS4000A_SWEEP_TYPE          sweepType,
        PS4000A_EXTRA_OPERATIONS    operation,
        uint32_t                    shots,
        uint32_t                    sweeps,
        PS4000A_SIGGEN_TRIG_TYPE    triggerType,
        PS4000A_SIGGEN_TRIG_SOURCE  triggerSource,
        int16_t                     extInThreshold
    ); """
ps4000a.make_symbol("_SetSigGenBuiltIn", "ps4000aSetSigGenBuiltIn", c_uint32,
                    [c_int16, c_int32, c_uint32, c_int32, c_double, c_double, c_double, c_double,
                     c_int32, c_int32, c_uint32, c_uint32, c_int32, c_int32, c_int16], doc)

doc = """ PICO_STATUS ps4000aSetSigGenPropertiesArbitrary
    (
        int16_t                     handle,
        uint32_t                    startDeltaPhase,
        uint32_t                    stopDeltaPhase,
        uint32_t                    deltaPhaseIncrement,
        uint32_t                    dwellCount,
        PS4000A_SWEEP_TYPE          sweepType,
        uint32_t                    shots,
        uint32_t                    sweeps,
        PS4000A_SIGGEN_TRIG_TYPE    triggerType,
        PS4000A_SIGGEN_TRIG_SOURCE  triggerSource,
        int16_t                     extInThreshold
    ); """
ps4000a.make_symbol("_SetSigGenPropertiesArbitrary", "ps4000aSetSigGenPropertiesArbitrary", c_uint32,
                    [c_int16, c_uint32, c_uint32, c_uint32, c_uint32, c_int32, c_uint32, c_uint32, c_int32, c_int32,
                     c_int16], doc)

doc = """ PICO_STATUS ps4000aSetSigGenPropertiesBuiltIn
    (
        int16_t                     handle,
        double                      startFrequency,
        double                      stopFrequency,
        double                      increment,
        double                      dwellTime,
        PS4000A_SWEEP_TYPE          sweepType,
        uint32_t                    shots,
        uint32_t                    sweeps,
        PS4000A_SIGGEN_TRIG_TYPE    triggerType,
        PS4000A_SIGGEN_TRIG_SOURCE  triggerSource,
        int16_t                     extInThreshold
    ); """
ps4000a.make_symbol("_SetSigGenPropertiesBuiltIn", "ps4000aSetSigGenPropertiesBuiltIn", c_uint32,
                    [c_int16, c_double, c_double, c_double, c_double, c_int32, c_uint32, c_uint32, c_int32, c_int32,
                     c_int16], doc)

doc = """ PICO_STATUS ps4000aSigGenFrequencyToPhase
    (
        int16_t             handle,
        double              frequency,
        PS4000A_INDEX_MODE  indexMode,
        uint32_t            bufferLength,
        uint32_t           *phase
    ); """
ps4000a.make_symbol("_SigGenFrequencyToPhase", "ps4000aSigGenFrequencyToPhase", c_uint32,
                    [c_int16, c_double, c_int32, c_uint32, c_void_p], doc)

doc = """ PICO_STATUS ps4000aSigGenArbitraryMinMaxValues
    (
        int16_t   handle,
        int16_t  *minArbitraryWaveformValue,
        int16_t  *maxArbitraryWaveformValue,
        uint32_t *minArbitraryWaveformSize,
        uint32_t *maxArbitraryWaveformSize
    ); """
ps4000a.make_symbol("_SigGenArbitraryMinMaxValues", "ps4000aSigGenArbitraryMinMaxValues", c_uint32,
                    [c_int16, c_void_p, c_void_p, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps4000aSigGenSoftwareControl
    (
        int16_t  handle,
        int16_t  state
    );"""
ps4000a.make_symbol("_SigGenSoftwareControl", "ps4000aSigGenSoftwareControl", c_uint32, [c_int16, c_int16], doc)

doc = """ PICO_STATUS ps4000aSetEts
    (
        int16_t           handle,
        PS4000A_ETS_MODE  mode,
        int16_t           etsCycles,
        int16_t           etsInterleave,
        int32_t          *sampleTimePicoseconds
    ); """
ps4000a.make_symbol("_SetEts", "ps4000aSetEts", c_uint32, [c_int16, c_int32, c_int16, c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps4000aSetTriggerChannelProperties
    (
        int16_t                             handle,
        PS4000A_TRIGGER_CHANNEL_PROPERTIES *channelProperties,
        int16_t                             nChannelProperties,
        int16_t                             auxOutputEnable,
        int32_t                             autoTriggerMilliseconds
    ); """
ps4000a.make_symbol("_SetTriggerChannelProperties", "ps4000aSetTriggerChannelProperties", c_uint32,
                    [c_int16, c_void_p, c_int16, c_int16, c_int32], doc)

doc = """ PICO_STATUS ps4000aSetTriggerChannelConditions
    (
        int16_t                  handle,
        PS4000A_CONDITION       *conditions,
        int16_t                  nConditions,
        PS4000A_CONDITIONS_INFO  info
    ); """
ps4000a.make_symbol("_SetTriggerChannelConditions", "ps4000aSetTriggerChannelConditions", c_uint32,
                    [c_int16, c_void_p, c_int16, c_int32], doc)

doc = """ PICO_STATUS ps4000aSetTriggerChannelDirections
    (
        int16_t            handle,
        PS4000A_DIRECTION *directions,
        int16_t            nDirections
    ); """
ps4000a.make_symbol("_SetTriggerChannelDirections", "ps4000aSetTriggerChannelDirections", c_uint32,
                    [c_int16, c_void_p, c_int16], doc)

doc = """ PICO_STATUS ps4000aSetSimpleTrigger
    (
        int16_t                      handle,
        int16_t                      enable,
        PS4000A_CHANNEL              source,
        int16_t                      threshold,
        PS4000A_THRESHOLD_DIRECTION  direction,
        uint32_t                     delay,
        int16_t                      autoTrigger_ms
    ); """
ps4000a.make_symbol("_SetSimpleTrigger", "ps4000aSetSimpleTrigger", c_uint32,
                    [c_int16, c_int16, c_int32, c_int16, c_int32, c_uint32, c_int16], doc)

doc = """ PICO_STATUS ps4000aSetTriggerDelay
    (
        int16_t   handle,
        uint32_t  delay
    ); """
ps4000a.make_symbol("_SetTriggerDelay", "ps4000aSetTriggerDelay", c_uint32, [c_int16, c_uint32], doc)

doc = """ PICO_STATUS ps4000aSetPulseWidthQualifierProperties
    (
        int16_t                      handle,
        PS4000A_THRESHOLD_DIRECTION  direction,
        uint32_t                     lower,
        uint32_t                     upper,
        PS4000A_PULSE_WIDTH_TYPE     type
    ); """
ps4000a.make_symbol("_SetPulseWidthQualifierProperties", "ps4000aSetPulseWidthQualifierProperties", c_uint32,
                    [c_int16, c_int32, c_uint32, c_uint32, c_int32], doc)

doc = """ PICO_STATUS ps4000aSetPulseWidthQualifierConditions
    (
        int16_t                  handle,
        PS4000A_CONDITION       *conditions,
        int16_t                  nConditions,
        PS4000A_CONDITIONS_INFO  info
    ); """
ps4000a.make_symbol("_SetPulseWidthQualifierConditions", "ps4000aSetPulseWidthQualifierConditions", c_uint32,
                    [c_int16, c_void_p, c_int16, c_int32], doc)

doc = """ PICO_STATUS ps4000aIsTriggerOrPulseWidthQualifierEnabled
    (
        int16_t  handle,
        int16_t *triggerEnabled,
        int16_t *pulseWidthQualifierEnabled
    ); """
ps4000a.make_symbol("_IsTriggerOrPulseWidthQualifierEnabled", "ps4000aIsTriggerOrPulseWidthQualifierEnabled", c_uint32,
                    [c_int16, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps4000aGetTriggerTimeOffset
    (
        int16_t             handle,
        uint32_t           *timeUpper,
        uint32_t           *timeLower,
        PS4000A_TIME_UNITS *timeUnits,
        uint32_t            segmentIndex
    ); """
ps4000a.make_symbol("_GetTriggerTimeOffset", "ps4000aGetTriggerTimeOffset", c_uint32,
                    [c_int16, c_void_p, c_void_p, c_void_p, c_uint32], doc)

doc = """ PICO_STATUS ps4000aGetTriggerTimeOffset64
    (
        int16_t             handle,
        int64_t            *time,
        PS4000A_TIME_UNITS *timeUnits,
        uint32_t            segmentIndex
    ); """
ps4000a.make_symbol("_GetTriggerTimeOffset64", "ps4000aGetTriggerTimeOffset64", c_uint32,
                    [c_int16, c_void_p, c_void_p, c_uint32], doc)

doc = """ PICO_STATUS ps4000aGetValuesTriggerTimeOffsetBulk
    (
        int16_t             handle,
        uint32_t           *timesUpper,
        uint32_t           *timesLower,
        PS4000A_TIME_UNITS *timeUnits,
        uint32_t            fromSegmentIndex,
        uint32_t            toSegmentIndex
    ); """
ps4000a.make_symbol("_GetValuesTriggerTimeOffsetBulk", "ps4000aGetValuesTriggerTimeOffsetBulk", c_uint32,
                    [c_int16, c_void_p, c_void_p, c_void_p, c_uint32, c_uint32], doc)

doc = """ PICO_STATUS ps4000aGetValuesTriggerTimeOffsetBulk64
    (
        int16_t             handle,
        int64_t            *times,
        PS4000A_TIME_UNITS *timeUnits,
        uint32_t            fromSegmentIndex,
        uint32_t            toSegmentIndex
    ); """
ps4000a.make_symbol("_GetValuesTriggerTimeOffsetBulk64", "ps4000aGetValuesTriggerTimeOffsetBulk64", c_uint32,
                    [c_int16, c_void_p, c_void_p, c_uint32, c_uint32], doc)

doc = """ PICO_STATUS ps4000aSetDataBuffers
    (
        int16_t             handle,
        PS4000A_CHANNEL     channel,
        int16_t            *bufferMax,
        int16_t            *bufferMin,
        int32_t             bufferLth,
        uint32_t            segmentIndex,
        PS4000A_RATIO_MODE  mode
    ); """
ps4000a.make_symbol("_SetDataBuffers", "ps4000aSetDataBuffers", c_uint32,
                    [c_int16, c_int32, c_void_p, c_void_p, c_int32, c_uint32, c_int32], doc)

doc = """ PICO_STATUS ps4000aSetDataBuffer
    (
        int16_t             handle,
        PS4000A_CHANNEL     channel,
        int16_t            *buffer,
        int32_t             bufferLth,
        uint32_t            segmentIndex,
        PS4000A_RATIO_MODE  mode
    ); """
ps4000a.make_symbol("_SetDataBuffer", "ps4000aSetDataBuffer", c_uint32,
                    [c_int16, c_int32, c_void_p, c_int32, c_uint32, c_int32], doc)

doc = """ PICO_STATUS ps4000aSetEtsTimeBuffer
    (
        int16_t  handle,
        int64_t *buffer,
        int32_t  bufferLth
    ); """
ps4000a.make_symbol("_SetEtsTimeBuffer", "ps4000aSetEtsTimeBuffer", c_uint32, [c_int16, c_void_p, c_int32], doc)

doc = """ PICO_STATUS ps4000aSetEtsTimeBuffers
    (
        int16_t   handle,
        uint32_t *timeUpper,
        uint32_t *timeLower,
        int32_t   bufferLth
    ); """
ps4000a.make_symbol("_SetEtsTimeBuffers", "ps4000aSetEtsTimeBuffers", c_uint32, [c_int16, c_void_p, c_void_p, c_int32],
                    doc)

doc = """ PICO_STATUS ps4000aIsReady
    (
        int16_t  handle,
        int16_t *ready
    ); """
ps4000a.make_symbol("_IsReady", "ps4000aIsReady", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps4000aRunBlock
    (
        int16_t            handle,
        int32_t            noOfPreTriggerSamples,
        int32_t            noOfPostTriggerSamples,
        uint32_t           timebase,
        int32_t           *timeIndisposedMs,
        uint32_t           segmentIndex,
        ps4000aBlockReady  lpReady,
        void              *pParameter
    ); """
ps4000a.make_symbol("_RunBlock", "ps4000aRunBlock", c_uint32,
                    [c_int16, c_int32, c_int32, c_uint32, c_void_p, c_uint32, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps4000aRunStreaming
    (
        int16_t             handle,
        uint32_t           *sampleInterval,
        PS4000A_TIME_UNITS  sampleIntervalTimeUnits,
        uint32_t            maxPreTriggerSamples,
        uint32_t            maxPostTriggerSamples,
        int16_t             autoStop,
        uint32_t            downSampleRatio,
        PS4000A_RATIO_MODE  downSampleRatioMode,
        uint32_t            overviewBufferSize
    ); """
ps4000a.make_symbol("_RunStreaming", "ps4000aRunStreaming", c_uint32,
                    [c_int16, c_void_p, c_int32, c_uint32, c_uint32, c_int16, c_uint32, c_int32, c_uint32], doc)

doc = """ PICO_STATUS ps4000aGetStreamingLatestValues
    (
        int16_t                handle,
        ps4000aStreamingReady  lpPs4000aReady,
        void                  *pParameter
    ); """
ps4000a.make_symbol("_GetStreamingLatestValues", "ps4000aGetStreamingLatestValues", c_uint32,
                    [c_int16, c_void_p, c_void_p], doc)

doc = """ void *ps4000aStreamingReady
    (
        int16_t   handle,
        int32_t   noOfSamples,
        uint32_t  startIndex,
        int16_t   overflow,
        uint32_t  triggerAt,
        int16_t   triggered,
        int16_t   autoStop,
        void     *pParameter
    );
    define a python function which accepts the correct arguments, and pass it to the constructor of this type.
    """

ps4000a.StreamingReadyType = C_CALLBACK_FUNCTION_FACTORY(None,
                                                         c_int16,
                                                         c_int32,
                                                         c_uint32,
                                                         c_int16,
                                                         c_uint32,
                                                         c_int16,
                                                         c_int16,
                                                         c_void_p)

ps4000a.StreamingReadyType.__doc__ = doc


doc = """ PICO_STATUS ps4000aNoOfStreamingValues
    (
        int16_t   handle,
        uint32_t *noOfValues
    ); """
ps4000a.make_symbol("_NoOfStreamingValues", "ps4000aNoOfStreamingValues", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps4000aGetMaxDownSampleRatio
    (
        int16_t             handle,
        uint32_t            noOfUnaggreatedSamples,
        uint32_t           *maxDownSampleRatio,
        PS4000A_RATIO_MODE  downSampleRatioMode,
        uint32_t            segmentIndex
    ); """
ps4000a.make_symbol("_GetMaxDownSampleRatio", "ps4000aGetMaxDownSampleRatio", c_uint32,
                    [c_int16, c_uint32, c_void_p, c_int32, c_uint32], doc)

doc = """ PICO_STATUS ps4000aGetValues
    (
        int16_t             handle,
        uint32_t            startIndex,
        uint32_t           *noOfSamples,
        uint32_t            downSampleRatio,
        PS4000A_RATIO_MODE  downSampleRatioMode,
        uint32_t            segmentIndex,
        int16_t            *overflow
    ); """
ps4000a.make_symbol("_GetValues", "ps4000aGetValues", c_uint32,
                    [c_int16, c_uint32, c_void_p, c_uint32, c_int32, c_uint32, c_void_p], doc)

doc = """ PICO_STATUS ps4000aGetValuesAsync
    (
        int16_t             handle,
        uint32_t            startIndex,
        uint32_t            noOfSamples,
        uint32_t            downSampleRatio,
        PS4000A_RATIO_MODE  downSampleRatioMode,
        uint32_t            segmentIndex,
        void               *lpDataReady,
        void               *pParameter
    ); """
ps4000a.make_symbol("_GetValuesAsync", "ps4000aGetValuesAsync", c_uint32,
                    [c_int16, c_uint32, c_uint32, c_uint32, c_int32, c_uint32, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps4000aGetValuesBulk
    (
        int16_t             handle,
        uint32_t           *noOfSamples,
        uint32_t            fromSegmentIndex,
        uint32_t            toSegmentIndex,
        uint32_t            downSampleRatio,
        PS4000A_RATIO_MODE  downSampleRatioMode,
        int16_t            *overflow
    ); """
ps4000a.make_symbol("_GetValuesBulk", "ps4000aGetValuesBulk", c_uint32,
                    [c_int16, c_void_p, c_uint32, c_uint32, c_uint32, c_int32, c_void_p], doc)

doc = """ PICO_STATUS ps4000aGetValuesOverlapped
    (
        int16_t             handle,
        uint32_t            startIndex,
        uint32_t           *noOfSamples,
        uint32_t            downSampleRatio,
        PS4000A_RATIO_MODE  downSampleRatioMode,
        uint32_t            segmentIndex,
        int16_t            *overflow
    ); """
ps4000a.make_symbol("_GetValuesOverlapped", "ps4000aGetValuesOverlapped", c_uint32,
                    [c_int16, c_uint32, c_void_p, c_uint32, c_int32, c_uint32, c_void_p], doc)

doc = """ PICO_STATUS ps4000aGetValuesOverlappedBulk
    (
        int16_t             handle,
        uint32_t            startIndex,
        uint32_t           *noOfSamples,
        uint32_t            downSampleRatio,
        PS4000A_RATIO_MODE  downSampleRatioMode,
        uint32_t            fromSegmentIndex,
        uint32_t            toSegmentIndex,
        int16_t            *overflow
    ); """
ps4000a.make_symbol("_GetValuesOverlappedBulk", "ps4000aGetValuesOverlappedBulk", c_uint32,
                    [c_int16, c_uint32, c_void_p, c_uint32, c_int32, c_uint32, c_uint32, c_void_p], doc)

doc = """ PICO_STATUS ps4000aEnumerateUnits
    (
        int16_t *count,
        int8_t  *serials,
        int16_t *serialLth
    ); """
ps4000a.make_symbol("_EnumerateUnits", "ps4000aEnumerateUnits", c_uint32, [c_void_p, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps4000aGetChannelInformation
    (
        int16_t               handle,
        PS4000A_CHANNEL_INFO  info,
        int32_t               probe,
        int32_t              *ranges,
        int32_t              *length,
        int32_t               channels
    ); """
ps4000a.make_symbol("_GetChannelInformation", "ps4000aGetChannelInformation", c_uint32,
                    [c_int16, c_int32, c_int32, c_void_p, c_void_p, c_int32], doc)

doc = """ PICO_STATUS ps4000aConnectDetect
    (
        int16_t                 handle,
        PS4000A_CONNECT_DETECT *sensor,
        int16_t                 nSensors
    ); """
ps4000a.make_symbol("_ConnectDetect", "ps4000aConnectDetect", c_uint32, [c_int16, c_void_p, c_int16], doc)

doc = """ PICO_STATUS ps4000aMaximumValue
    (
        int16_t  handle,
        int16_t *value
    ); """
ps4000a.make_symbol("_MaximumValue", "ps4000aMaximumValue", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps4000aMinimumValue
    (
        int16_t		handle,
        int16_t * value
    ); """
ps4000a.make_symbol("_MinimumValue", "ps4000aMinimumValue", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps4000aGetAnalogueOffset
    (
        int16_t           handle,
        PS4000A_RANGE     range,
        PS4000A_COUPLING  coupling,
        float            *maximumVoltage,
        float            *minimumVoltage
    ); """
ps4000a.make_symbol("_GetAnalogueOffset", "ps4000aGetAnalogueOffset", c_uint32,
                    [c_int16, c_int32, c_int32, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps4000aGetMaxSegments
    (
        int16_t   handle,
        uint32_t *maxSegments
    ); """
ps4000a.make_symbol("_GetMaxSegments", "ps4000aGetMaxSegments", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps4000aChangePowerSource
    (
        int16_t      handle,
        PICO_STATUS  powerState
    ); """
ps4000a.make_symbol("_ChangePowerSource", "ps4000aChangePowerSource", c_uint32, [c_int16, c_uint32], doc)

doc = """ PICO_STATUS ps4000aCurrentPowerSource
    (
        int16_t  handle
    ); """
ps4000a.make_symbol("_CurrentPowerSource", "ps4000aCurrentPowerSource", c_uint32, [c_int16, ], doc)

doc = """ PICO_STATUS ps4000aStop
    (
        int16_t  handle
    ); """
ps4000a.make_symbol("_Stop", "ps4000aStop", c_uint32, [c_int16, ], doc)

doc = """ PICO_STATUS ps4000aPingUnit
    (
        int16_t  handle
    ); """
ps4000a.make_symbol("_PingUnit", "ps4000aPingUnit", c_uint32, [c_int16, ], doc)

doc = """ PICO_STATUS ps4000aSetNoOfCaptures
    (
        int16_t   handle,
        uint32_t  nCaptures
    ); """
ps4000a.make_symbol("_SetNoOfCaptures", "ps4000aSetNoOfCaptures", c_uint32, [c_int16, c_uint32], doc)

doc = """ PICO_STATUS ps4000aGetNoOfCaptures
    (
        int16_t   handle,
        uint32_t *nCaptures
    ); """
ps4000a.make_symbol("_GetNoOfCaptures", "ps4000aGetNoOfCaptures", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps4000aGetNoOfProcessedCaptures
    (
        int16_t   handle,
        uint32_t *nProcessedCaptures
    ); """
ps4000a.make_symbol("_GetNoOfProcessedCaptures", "ps4000aGetNoOfProcessedCaptures", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps4000aDeviceMetaData
    (
        int16_t                 handle,
        int8_t                 *settings,
        int32_t                *nSettingsLength,
        PS4000A_META_TYPE       type,
        PS4000A_META_OPERATION  operation,
        PS4000A_META_FORMAT     format
    ); """
ps4000a.make_symbol("_DeviceMetaData", "ps4000aDeviceMetaData", c_uint32,
                    [c_int16, c_void_p, c_void_p, c_int32, c_int32, c_int32], doc)

doc = """ PICO_STATUS ps4000aGetString
    (
        int16_t            handle,
        PICO_STRING_VALUE  stringValue,
        int8_t            *string,
        int32_t           *stringLength
    ); """
ps4000a.make_symbol("_GetString", "ps4000aGetString", c_uint32, [c_int16, c_int32, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps4000aGetCommonModeOverflow
    (
        int16_t   handle,
        uint16_t *overflow
    ); """
ps4000a.make_symbol("_GetCommonModeOverflow", "ps4000aGetCommonModeOverflow", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps4000aSetFrequencyCounter
    (
        int16_t                          handle,
        PS4000A_CHANNEL                  channel,
        int16_t                          enabled,
        PS4000A_FREQUENCY_COUNTER_RANGE  range,
        int16_t                          thresholdMajor,
        int16_t                          thresholdMinor
    ); """
ps4000a.make_symbol("_SetFrequencyCounter", "ps4000aSetFrequencyCounter", c_uint32,
                    [c_int16, c_int32, c_int16, c_int32, c_int16, c_int16], doc)

doc = """ PICO_STATUS ps4000aOpenUnitWithResolution
    (
	    int16_t    *handle,
		int8_t     *serial,
		PS4000A_DEVICE_RESOLUTION    resolution
	); """
ps4000a.make_symbol("_OpenUnitWithResolution", "ps4000aOpenUnitWithResolution", c_uint32, [c_void_p, c_void_p, c_int32], doc)

doc = """ PICO_STATUS ps4000aGetDeviceResolution
    (
	    int16_t    handle,
		PS4000A_DEVICE_RESOLUTION    *resolution
	); """
ps4000a.make_symbol("_GetResolution", "ps4000aGetDeviceResolution", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps4000aSetDeviceResolution
    (
	    int16_t    handle,
		PS4000A_DEVICE_RESOLUTION    resolution
	); """
ps4000a.make_symbol("_SetResolution", "ps4000aSetDeviceResolution", c_uint32, [c_int16, c_int32], doc)



#
# Copyright (C) 2019 Pico Technology Ltd. See LICENSE file for terms.
#
# PL1000 SINGLE MODE EXAMPLE
# This example opens a pl1000 device, sets up the device for capturing data from channel 1.
# Then this example collect a sample from channel 1 and displays it on the console.

import ctypes
import numpy as np
from picosdk.pl1000 import pl1000 as pl
import matplotlib.pyplot as plt
from picosdk.functions import adc2mVpl1000, assert_pico_ok
from time import sleep

# Create chandle and status ready for use
chandle = ctypes.c_int16()
status = {}

# open PicoLog 1000 device
status["openUnit"] = pl.pl1000OpenUnit(ctypes.byref(chandle))
assert_pico_ok(status["openUnit"])

# set sampling interval
usForBlock = ctypes.c_uint32(10000000)
noOfValues = ctypes.c_uint32(1000000)
channels = ctypes.c_int16(1)

status["setInterval"] = pl.pl1000SetInterval(chandle, ctypes.byref(usForBlock), noOfValues, ctypes.byref(channels), 1)
assert_pico_ok(status["setInterval"])

# start streaming
mode = pl.PL1000_BLOCK_METHOD["BM_STREAM"]
status["run"] = pl.pl1000Run(chandle, 1000000, mode)
assert_pico_ok(status["run"])

sleep(usForBlock.value / 1000000)

values = (ctypes.c_uint16 * noOfValues.value)()
oveflow = ctypes.c_uint16()

status["getValues"] = pl.pl1000GetValues(chandle, ctypes.byref(values), ctypes.byref(noOfValues), ctypes.byref(oveflow), None)
assert_pico_ok(status["getValues"])

# convert ADC counts data to mV
maxADC = ctypes.c_uint16()
status["maxValue"] = pl.pl1000MaxValue(chandle, ctypes.byref(maxADC))
assert_pico_ok(status["maxValue"])
inputRange = 2500
mVValues =  adc2mVpl1000(values, inputRange, maxADC)

# create time data
interval = (0.01 * usForBlock.value)/(noOfValues.value * 1)

timeMs = np.linspace(0, (len(mVValues) -1) * interval, len(mVValues))

# plot data

plt.plot(timeMs, mVValues[:])
plt.xlabel('Time (ms)')
plt.ylabel('Voltage (mV)')
plt.show()

# close PicoLog 1000 device
status["closeUnit"] = pl.pl1000CloseUnit(chandle)
assert_pico_ok(status["closeUnit"])

# display status returns
print(status)



#
# Copyright (C) 2020 Pico Technology Ltd. See LICENSE file for terms.
#
# PS6000 GET INFO EXAMPLE
# This example opens a 6000 driver device, gets the variant info and closes the scope.

import ctypes
import numpy as np
from picosdk.ps6000 import ps6000 as ps
import matplotlib.pyplot as plt
from picosdk.functions import adc2mV, assert_pico_ok

# Create chandle and status ready for use
chandle = ctypes.c_int16()
status = {}

# Open 6000 series PicoScope
# Returns handle to chandle for use in future API functions
status["openunit"] = ps.ps6000OpenUnit(ctypes.byref(chandle), None)
assert_pico_ok(status["openunit"])

# Get Info from scope
string = (ctypes.c_char * 40)()
stringLength = ctypes.c_int16(40)
requiredSize = ctypes.c_int16(40)
info = ps.PICO_INFO["PICO_VARIANT_INFO"]
status["getInfo"] = ps.ps6000GetUnitInfo(chandle, ctypes.byref(string),stringLength, ctypes.byref(requiredSize), info)
assert_pico_ok(status["getInfo"])

print(string.value)

# Close unitDisconnect the scope
# handle = chandle
ps.ps6000CloseUnit(chandle)

# display status returns
print(status)


#
# Copyright (C) 2018-2022 Pico Technology Ltd. See LICENSE file for terms.
#
# PS5000A BLOCK MODE EXAMPLE
# This example opens a 5000a driver device, sets up two channels and a trigger then collects a block of data.
# This data is then plotted as mV against time in ns.

import ctypes
import numpy as np
from picosdk.ps5000a import ps5000a as ps
import matplotlib.pyplot as plt
from picosdk.functions import adc2mV, assert_pico_ok, mV2adc

# Create chandle and status ready for use
chandle = ctypes.c_int16()
status = {}

# Open 5000 series PicoScope
# Resolution set to 12 Bit
resolution =ps.PS5000A_DEVICE_RESOLUTION["PS5000A_DR_12BIT"]
# Returns handle to chandle for use in future API functions
status["openunit"] = ps.ps5000aOpenUnit(ctypes.byref(chandle), None, resolution)

try:
    assert_pico_ok(status["openunit"])
except: # PicoNotOkError:

    powerStatus = status["openunit"]

    if powerStatus == 286:
        status["changePowerSource"] = ps.ps5000aChangePowerSource(chandle, powerStatus)
    elif powerStatus == 282:
        status["changePowerSource"] = ps.ps5000aChangePowerSource(chandle, powerStatus)
    else:
        raise

    assert_pico_ok(status["changePowerSource"])

# Set up channel A
# handle = chandle
channel = ps.PS5000A_CHANNEL["PS5000A_CHANNEL_A"]
# enabled = 1
coupling_type = ps.PS5000A_COUPLING["PS5000A_DC"]
chARange = ps.PS5000A_RANGE["PS5000A_20V"]
# analogue offset = 0 V
status["setChA"] = ps.ps5000aSetChannel(chandle, channel, 1, coupling_type, chARange, 0)
assert_pico_ok(status["setChA"])

# Set up channel B
# handle = chandle
channel = ps.PS5000A_CHANNEL["PS5000A_CHANNEL_B"]
# enabled = 1
# coupling_type = ps.PS5000A_COUPLING["PS5000A_DC"]
chBRange = ps.PS5000A_RANGE["PS5000A_2V"]
# analogue offset = 0 V
status["setChB"] = ps.ps5000aSetChannel(chandle, channel, 1, coupling_type, chBRange, 0)
assert_pico_ok(status["setChB"])

# find maximum ADC count value
# handle = chandle
# pointer to value = ctypes.byref(maxADC)
maxADC = ctypes.c_int16()
status["maximumValue"] = ps.ps5000aMaximumValue(chandle, ctypes.byref(maxADC))
assert_pico_ok(status["maximumValue"])

# Set up single trigger
# handle = chandle
# enabled = 1
source = ps.PS5000A_CHANNEL["PS5000A_CHANNEL_A"]
threshold = int(mV2adc(500,chARange, maxADC))
# direction = PS5000A_RISING = 2
# delay = 0 s
# auto Trigger = 1000 ms
status["trigger"] = ps.ps5000aSetSimpleTrigger(chandle, 1, source, threshold, 2, 0, 1000)
assert_pico_ok(status["trigger"])

# Set number of pre and post trigger samples to be collected
preTriggerSamples = 2500
postTriggerSamples = 2500
maxSamples = preTriggerSamples + postTriggerSamples

# Get timebase information
# Warning: When using this example it may not be possible to access all Timebases as all channels are enabled by default when opening the scope.  
# To access these Timebases, set any unused analogue channels to off.
# handle = chandle
timebase = 8
# noSamples = maxSamples
# pointer to timeIntervalNanoseconds = ctypes.byref(timeIntervalns)
# pointer to maxSamples = ctypes.byref(returnedMaxSamples)
# segment index = 0
timeIntervalns = ctypes.c_float()
returnedMaxSamples = ctypes.c_int32()
status["getTimebase2"] = ps.ps5000aGetTimebase2(chandle, timebase, maxSamples, ctypes.byref(timeIntervalns), ctypes.byref(returnedMaxSamples), 0)
assert_pico_ok(status["getTimebase2"])

# Run block capture
# handle = chandle
# number of pre-trigger samples = preTriggerSamples
# number of post-trigger samples = PostTriggerSamples
# timebase = 8 = 80 ns (see Programmer's guide for mre information on timebases)
# time indisposed ms = None (not needed in the example)
# segment index = 0
# lpReady = None (using ps5000aIsReady rather than ps5000aBlockReady)
# pParameter = None
status["runBlock"] = ps.ps5000aRunBlock(chandle, preTriggerSamples, postTriggerSamples, timebase, None, 0, None, None)
assert_pico_ok(status["runBlock"])

# Check for data collection to finish using ps5000aIsReady
ready = ctypes.c_int16(0)
check = ctypes.c_int16(0)
while ready.value == check.value:
    status["isReady"] = ps.ps5000aIsReady(chandle, ctypes.byref(ready))


# Create buffers ready for assigning pointers for data collection
bufferAMax = (ctypes.c_int16 * maxSamples)()
bufferAMin = (ctypes.c_int16 * maxSamples)() # used for downsampling which isn't in the scope of this example
bufferBMax = (ctypes.c_int16 * maxSamples)()
bufferBMin = (ctypes.c_int16 * maxSamples)() # used for downsampling which isn't in the scope of this example

# Set data buffer location for data collection from channel A
# handle = chandle
source = ps.PS5000A_CHANNEL["PS5000A_CHANNEL_A"]
# pointer to buffer max = ctypes.byref(bufferAMax)
# pointer to buffer min = ctypes.byref(bufferAMin)
# buffer length = maxSamples
# segment index = 0
# ratio mode = PS5000A_RATIO_MODE_NONE = 0
status["setDataBuffersA"] = ps.ps5000aSetDataBuffers(chandle, source, ctypes.byref(bufferAMax), ctypes.byref(bufferAMin), maxSamples, 0, 0)
assert_pico_ok(status["setDataBuffersA"])

# Set data buffer location for data collection from channel B
# handle = chandle
source = ps.PS5000A_CHANNEL["PS5000A_CHANNEL_B"]
# pointer to buffer max = ctypes.byref(bufferBMax)
# pointer to buffer min = ctypes.byref(bufferBMin)
# buffer length = maxSamples
# segment index = 0
# ratio mode = PS5000A_RATIO_MODE_NONE = 0
status["setDataBuffersB"] = ps.ps5000aSetDataBuffers(chandle, source, ctypes.byref(bufferBMax), ctypes.byref(bufferBMin), maxSamples, 0, 0)
assert_pico_ok(status["setDataBuffersB"])

# create overflow loaction
overflow = ctypes.c_int16()
# create converted type maxSamples
cmaxSamples = ctypes.c_int32(maxSamples)

# Retried data from scope to buffers assigned above
# handle = chandle
# start index = 0
# pointer to number of samples = ctypes.byref(cmaxSamples)
# downsample ratio = 0
# downsample ratio mode = PS5000A_RATIO_MODE_NONE
# pointer to overflow = ctypes.byref(overflow))
status["getValues"] = ps.ps5000aGetValues(chandle, 0, ctypes.byref(cmaxSamples), 0, 0, 0, ctypes.byref(overflow))
assert_pico_ok(status["getValues"])


# convert ADC counts data to mV
adc2mVChAMax =  adc2mV(bufferAMax, chARange, maxADC)
adc2mVChBMax =  adc2mV(bufferBMax, chBRange, maxADC)

# Create time data
time = np.linspace(0, (cmaxSamples.value - 1) * timeIntervalns.value, cmaxSamples.value)

# plot data from channel A and B
plt.plot(time, adc2mVChAMax[:])
plt.plot(time, adc2mVChBMax[:])
plt.xlabel('Time (ns)')
plt.ylabel('Voltage (mV)')
plt.show()

# Stop the scope
# handle = chandle
status["stop"] = ps.ps5000aStop(chandle)
assert_pico_ok(status["stop"])

# Close unit Disconnect the scope
# handle = chandle
status["close"]=ps.ps5000aCloseUnit(chandle)
assert_pico_ok(status["close"])

# display status returns
print(status)


#
# Copyright (C) 2019 Pico Technology Ltd. See LICENSE file for terms.
#
# PL1000 SINGLE MODE EXAMPLE
# This example opens a pl1000 device, sets up the device for capturing data from channel 1.
# Then this example collect a sample from channel 1 and displays it on the console.

import ctypes
import numpy as np
from picosdk.pl1000 import pl1000 as pl
import matplotlib.pyplot as plt
from picosdk.functions import adc2mV, assert_pico_ok

# Create chandle and status ready for use
chandle = ctypes.c_int16()
status = {}

# open PicoLog 1000 device
status["openUnit"] = pl.pl1000OpenUnit(ctypes.byref(chandle))
assert_pico_ok(status["openUnit"])

value = ctypes.c_int16()
# get a single ADC count value from channel 1
status["getSingle"] = pl.pl1000GetSingle(chandle, pl.PL1000Inputs["PL1000_CHANNEL_1"], ctypes.byref(value))
assert_pico_ok(status["getSingle"])
print(value.value)

# close PicoLog 1000 device
status["closeUnit"] = pl.pl1000CloseUnit(chandle)
assert_pico_ok(status["closeUnit"])

# display status returns
print(status)



#
# Copyright (C) 2015-2018 Pico Technology Ltd. See LICENSE file for terms.
#
"""
This is a Python module defining the functions from the ps3000.h C header file
for PicoScope 3000 Series oscilloscopes using the ps3000 driver API functions.
"""

from ctypes import *
from picosdk.library import Library
from picosdk.constants import make_enum


class Ps3000lib(Library):
    def __init__(self):
        super(Ps3000lib, self).__init__("ps3000")


ps3000 = Ps3000lib()

ps3000.PS3000_CHANNEL = make_enum([
    "PS3000_CHANNEL_A",
    "PS3000_CHANNEL_B",
    "PS3000_CHANNEL_C",
    "PS3000_CHANNEL_D",
])

# use the last character, i.e. the channel name:
ps3000.PICO_CHANNEL = {k[-1]: v for k, v in ps3000.PS3000_CHANNEL.items()}

# This field is passed to the driver as a boolean, not an enum.
ps3000.PICO_COUPLING = {
    'AC': 0,
    'DC': 1
}

ps3000.PS3000_VOLTAGE_RANGE = {
    'PS3000_20MV':  1,
    'PS3000_50MV':  2,
    'PS3000_100MV': 3,
    'PS3000_200MV': 4,
    'PS3000_500MV': 5,
    'PS3000_1V':    6,
    'PS3000_2V':    7,
    'PS3000_5V':    8,
    'PS3000_10V':   9,
    'PS3000_20V':   10,
    'PS3000_50V':   11,
    'PS3000_100V':  12,
    'PS3000_200V':  13,
    'PS3000_400V':  14,
}

# float voltage value max (multiplier for output voltages). Parse the value in the constant name.
ps3000.PICO_VOLTAGE_RANGE = {
    v: float(k.split('_')[1][:-1]) if k[-2] != 'M' else (0.001 * float(k.split('_')[1][:-2]))
    for k, v in ps3000.PS3000_VOLTAGE_RANGE.items()
}

doc = """ int16_t ps3000_open_unit
    (
        void
    ); """
ps3000.make_symbol("_open_unit", "ps3000_open_unit", c_int16, [], doc)

doc = """ int16_t ps3000_get_unit_info
    (
        int16_t  handle,
        int8_t  *string,
        int16_t  string_length,
        int16_t  line
    ); """
ps3000.make_symbol("_get_unit_info", "ps3000_get_unit_info", c_int16, [c_int16, c_char_p, c_int16, c_int16], doc)

doc = """ int16_t ps3000_flash_led
    (
        int16_t handle
    ); """
ps3000.make_symbol("_flash_led", "ps3000_flash_led", c_int16, [c_int16, ], doc)

doc = """ int16_t ps3000_close_unit
    (
        int16_t handle
    ); """
ps3000.make_symbol("_close_unit", "ps3000_close_unit", c_int16, [c_int16, ], doc)

doc = """ int16_t ps3000_set_channel
    (
        int16_t  handle,
        int16_t  channel,
        int16_t  enabled,
        int16_t  dc,
        int16_t  range
    ); """
ps3000.make_symbol("_set_channel", "ps3000_set_channel", c_int16, [c_int16, c_int16, c_int16, c_int16, c_int16], doc)

doc = """ int16_t ps3000_get_timebase
    (
        int16_t  handle,
        int16_t  timebase,
        int32_t  no_of_samples,
        int32_t *time_interval,
        int16_t *time_units,
        int16_t  oversample,
        int32_t *max_samples
    ); """
ps3000.make_symbol("_get_timebase", "ps3000_get_timebase", c_int16,
                   [c_int16, c_int16, c_int32, c_void_p, c_void_p, c_int16, c_void_p], doc)

doc = """ int32_t ps3000_set_siggen
    (
        int16_t  handle,
        int16_t  wave_type,
        int32_t  start_frequency,
        int32_t  stop_frequency,
        float    increment,
        int16_t  dwell_time,
        int16_t  repeat,
        int16_t  dual_slope
    ); """
ps3000.make_symbol("_set_siggen", "ps3000_set_siggen", c_int32,
                   [c_int16, c_int16, c_int32, c_int32, c_float, c_int16, c_int16, c_int16], doc)

doc = """ int32_t ps3000_set_ets
    (
        int16_t  handle,
        int16_t  mode,
        int16_t  ets_cycles,
        int16_t  ets_interleave
    ); """
ps3000.make_symbol("_set_ets", "ps3000_set_ets", c_int32, [c_int16, c_int16, c_int16, c_int16], doc)

doc = """ int16_t ps3000_set_trigger
    (
        int16_t  handle,
        int16_t  source,
        int16_t  threshold,
        int16_t  direction,
        int16_t  delay,
        int16_t  auto_trigger_ms
    ); """
ps3000.make_symbol("_set_trigger", "ps3000_set_trigger", c_int16,
                   [c_int16, c_int16, c_int16, c_int16, c_int16, c_int16], doc)

doc = """ int16_t ps3000_set_trigger2
    (
        int16_t  handle,
        int16_t  source,
        int16_t  threshold,
        int16_t  direction,
        float    delay,
        int16_t  auto_trigger_ms
    ); """
ps3000.make_symbol("_set_trigger2", "ps3000_set_trigger2", c_int16,
                   [c_int16, c_int16, c_int16, c_int16, c_float, c_int16], doc)

doc = """ int16_t ps3000_run_block
    (
        int16_t handle,
        int32_t  no_of_values,
        int16_t  timebase,
        int16_t  oversample,
        int32_t * time_indisposed_ms
    ); """
ps3000.make_symbol("_run_block", "ps3000_run_block", c_int16, [c_int16, c_int32, c_int16, c_int16, c_void_p], doc)

doc = """ int16_t ps3000_run_streaming
    (
        int16_t  handle,
        int16_t  sample_interval_ms,
        int32_t  max_samples,
        int16_t  windowed
    ); """
ps3000.make_symbol("_run_streaming", "ps3000_run_streaming", c_int16, [c_int16, c_int16, c_int32, c_int16], doc)

doc = """ int16_t ps3000_run_streaming_ns
    (
        int16_t            handle,
        uint32_t           sample_interval,
        PS3000_TIME_UNITS  time_units,
        uint32_t           max_samples,
        int16_t            auto_stop,
        uint32_t           noOfSamplesPerAggregate,
        uint32_t           overview_buffer_size
    ); """
ps3000.make_symbol("_run_streaming_ns", "ps3000_run_streaming_ns", c_int16,
                   [c_int16, c_uint32, c_int32, c_uint32, c_int16, c_uint32, c_uint32], doc)

doc = """ int16_t ps3000_ready
    (
        int16_t  handle
    ); """
ps3000.make_symbol("_ready", "ps3000_ready", c_int16, [c_int16, ], doc)

doc = """ int16_t ps3000_stop
    (
        int16_t  handle
    ); """
ps3000.make_symbol("_stop", "ps3000_stop", c_int16, [c_int16, ], doc)

doc = """ int32_t ps3000_get_values
    (
        int16_t  handle,
        int16_t *buffer_a,
        int16_t *buffer_b,
        int16_t *buffer_c,
        int16_t *buffer_d,
        int16_t *overflow,
        int32_t  no_of_values
    ); """
ps3000.make_symbol("_get_values", "ps3000_get_values", c_int32,
                   [c_int16, c_void_p, c_void_p, c_void_p, c_void_p, c_void_p, c_int32], doc)

doc = """ void ps3000_release_stream_buffer
    (
        int16_t  handle
    ); """
ps3000.make_symbol("_release_stream_buffer", "ps3000_release_stream_buffer", None, [c_int16, ], doc)

doc = """ int32_t ps3000_get_times_and_values
    (
        int16_t  handle,
        int32_t *times,
        int16_t *buffer_a,
        int16_t *buffer_b,
        int16_t *buffer_c,
        int16_t *buffer_d,
        int16_t *overflow,
        int16_t  time_units,
        int32_t  no_of_values
    ); """
ps3000.make_symbol("_get_times_and_values", "ps3000_get_times_and_values", c_int32,
                   [c_int16, c_void_p, c_void_p, c_void_p, c_void_p, c_void_p, c_void_p, c_int16, c_int32], doc)

doc = """ int16_t ps3000_open_unit_async
    (
        void
    ); """
ps3000.make_symbol("_open_unit_async", "ps3000_open_unit_async", c_int16, [], doc)

doc = """ int16_t ps3000_open_unit_progress
    (
        int16_t *handle,
        int16_t *progress_percent
    ); """
ps3000.make_symbol("_open_unit_progress", "ps3000_open_unit_progress", c_int16, [c_void_p, c_void_p], doc)

doc = """ int16_t ps3000_streaming_ns_get_interval_stateless
    (
        int16_t   handle,
        int16_t   nChannels,
        uint32_t *sample_interval
    ); """
ps3000.make_symbol("_streaming_ns_get_interval_stateless", "ps3000_streaming_ns_get_interval_stateless", c_int16,
                   [c_int16, c_int16, c_void_p], doc)

doc = """ int16_t ps3000_get_streaming_last_values
    (
        int16_t  handle,
        GetOverviewBuffersMaxMin
    ); """
ps3000.make_symbol("_get_streaming_last_values", "ps3000_get_streaming_last_values", c_int16, [c_int16, c_void_p], doc)

doc = """ int16_t ps3000_overview_buffer_status
    (
        int16_t  handle,
        int16_t *previous_buffer_overrun
    ); """
ps3000.make_symbol("_overview_buffer_status", "ps3000_overview_buffer_status", c_int16, [c_int16, c_void_p], doc)

doc = """ uint32_t ps3000_get_streaming_values
    (
        int16_t  handle,
        double   *start_time,
        int16_t  *pbuffer_a_max,
        int16_t  *pbuffer_a_min,
        int16_t  *pbuffer_b_max,
        int16_t  *pbuffer_b_min,
        int16_t  *pbuffer_c_max,
        int16_t  *pbuffer_c_min,
        int16_t  *pbuffer_d_max,
        int16_t  *pbuffer_d_min,
        int16_t  *overflow,
        uint32_t *triggerAt,
        int16_t  *triggered,
        uint32_t  no_of_values,
        uint32_t  noOfSamplesPerAggregate
    ); """
ps3000.make_symbol("_get_streaming_values", "ps3000_get_streaming_values", c_uint32,
                   [c_int16, c_void_p,
                    c_void_p, c_void_p, c_void_p, c_void_p, c_void_p, c_void_p, c_void_p, c_void_p,
                    c_void_p, c_void_p, c_void_p, c_uint32, c_uint32], doc)

doc = """ uint32_t ps3000_get_streaming_values_no_aggregation
    (
        int16_t handle,
        double *start_time,
        int16_t * pbuffer_a,
        int16_t * pbuffer_b,
        int16_t * pbuffer_c,
        int16_t * pbuffer_d,
        int16_t * overflow,
        uint32_t * triggerAt,
        int16_t * trigger,
        uint32_t no_of_values
    ); """
ps3000.make_symbol("_get_streaming_values_no_aggregation", "ps3000_get_streaming_values_no_aggregation", c_uint32,
                   [c_int16, c_void_p,
                    c_void_p, c_void_p, c_void_p, c_void_p,
                    c_void_p, c_void_p, c_void_p, c_uint32], doc)

doc = """ int16_t ps3000_save_streaming_data
    (
        int16_t               handle,
        PS3000_CALLBACK_FUNC  lpCallbackFunc,
        int16_t              *dataBuffers,
        int16_t               dataBufferSize
    ); """
ps3000.make_symbol("_save_streaming_data", "ps3000_save_streaming_data", c_int16,
                   [c_int16, c_void_p, c_void_p, c_int16], doc)

doc = """ int16_t ps3000SetAdvTriggerChannelProperties
    (
        int16_t                            handle,
        PS3000_TRIGGER_CHANNEL_PROPERTIES *channelProperties,
        int16_t                            nChannelProperties,
        int32_t                            autoTriggerMilliseconds
    ); """
ps3000.make_symbol("_SetAdvTriggerChannelProperties", "ps3000SetAdvTriggerChannelProperties", c_int16,
                   [c_int16, c_void_p, c_int16, c_int32], doc)

doc = """ int16_t ps3000SetAdvTriggerChannelConditions
    (
        int16_t                    handle,
        PS3000_TRIGGER_CONDITIONS *conditions,
        int16_t                    nConditions
    ); """
ps3000.make_symbol("_SetAdvTriggerChannelConditions", "ps3000SetAdvTriggerChannelConditions", c_int16,
                   [c_int16, c_void_p, c_int16], doc)

doc = """ int16_t ps3000SetAdvTriggerChannelDirections
    (
        int16_t                     handle,
        PS3000_THRESHOLD_DIRECTION  channelA,
        PS3000_THRESHOLD_DIRECTION  channelB,
        PS3000_THRESHOLD_DIRECTION  channelC,
        PS3000_THRESHOLD_DIRECTION  channelD,
        PS3000_THRESHOLD_DIRECTION  ext
    ); """
ps3000.make_symbol("_SetAdvTriggerChannelDirections", "ps3000SetAdvTriggerChannelDirections", c_int16,
                   [c_int16, c_int32, c_int32, c_int32, c_int32, c_int32], doc)

doc = """ int16_t ps3000SetPulseWidthQualifier
    (
        int16_t                     handle,
        PS3000_PWQ_CONDITIONS      *conditions,
        int16_t                     nConditions,
        PS3000_THRESHOLD_DIRECTION  direction,
        uint32_t                    lower,
        uint32_t                    upper,
        PS3000_PULSE_WIDTH_TYPE     type
    ); """
ps3000.make_symbol("_SetPulseWidthQualifier", "ps3000SetPulseWidthQualifier", c_int16,
                   [c_int16, c_void_p, c_int16, c_int32, c_uint32, c_uint32, c_int32], doc)

doc = """ int16_t ps3000SetAdvTriggerDelay
    (
        int16_t   handle,
        uint32_t  delay,
        float     preTriggerDelay
    ); """
ps3000.make_symbol("_SetAdvTriggerDelay", "ps3000SetAdvTriggerDelay", c_int16, [c_int16, c_uint32, c_float], doc)

doc = """ int16_t ps3000PingUnit
    (
        int16_t  handle
    ); """
ps3000.make_symbol("_PingUnit", "ps3000PingUnit", c_int16, [c_int16, ], doc)

ps3000.PICO_INFO = {k: v for k, v in ps3000.PICO_INFO.items() if v <= 0x00000005}
ps3000.PICO_INFO["PICO_ERROR_CODE"] = 0x00000006



#
# Copyright (C) 2018 Pico Technology Ltd. See LICENSE file for terms.
#
"""
Unit tests for the wrapper functions for computing and validating timebases
"""

from __future__ import print_function

from test.test_helpers import DriverTest, drivers_with_device_connected
import unittest
from picosdk.library import TimebaseInfo
from picosdk.errors import InvalidTimebaseError, NoValidTimebaseForOptionsError
from picosdk.device import Device, TimebaseOptions
import math


class FindTimebaseTest(DriverTest):
    def assertValidTimebases(self, input_config, output_info):
        if input_config.max_time_interval is not None:
            self.assertLessEqual(output_info.time_interval, input_config.max_time_interval)
        if input_config.no_of_samples is not None:
            self.assertGreaterEqual(output_info.max_samples, input_config.no_of_samples)
        if input_config.min_collection_time is not None:
            self.assertGreaterEqual(output_info.time_interval * output_info.max_samples,
                                    input_config.min_collection_time)

    def test_find_timebase_success(self):
        """test_find_timebase_success
        note: test assumes you have set test_helpers.drivers_with_device_connected"""
        if not drivers_with_device_connected:
            return
        drivers_to_use = drivers_with_device_connected

        def test(driver):

            with driver.open_unit() as device:
                five_milliseconds = 0.005
                config = TimebaseOptions(max_time_interval=five_milliseconds,
                                         no_of_samples=None,
                                         min_collection_time=five_milliseconds * 30,
                                         oversample=1)
                timebase_info = device.find_timebase(config)

            self.assertValidTimebases(config, timebase_info)

        self.run_snippet_and_count_problems(drivers_to_use, test)

    def test_find_timebase_terrasample(self):
        """test_find_timebase_terrasample
        note: test assumes you have set test_helpers.drivers_with_device_connected"""
        if not drivers_with_device_connected:
            return
        drivers_to_use = drivers_with_device_connected

        def test(driver):
            with driver.open_unit() as device:
                threw = False
                try:
                    # we ask for a config which requires 1TS (one terra-sample) of memory on device.
                    smallest_timebase = 1.e-8
                    one_terra_sample = 1.e12
                    duration = smallest_timebase * one_terra_sample
                    config = TimebaseOptions(max_time_interval=smallest_timebase,
                                             no_of_samples=one_terra_sample,
                                             min_collection_time=duration)
                    timebase_info = device.find_timebase(config)
                except NoValidTimebaseForOptionsError:
                    threw = True
                if not threw:
                    return "didn't throw an NoValidTimebaseForOptionsError."

        self.run_snippet_and_count_problems(drivers_to_use, test)

    def test_find_timebase_impossible(self):
        """test_find_timebase_impossible
        note: test assumes you have set test_helpers.drivers_with_device_connected"""
        if not drivers_with_device_connected:
            return
        drivers_to_use = drivers_with_device_connected

        def test(driver):
            with driver.open_unit() as device:
                threw = False
                try:
                    # we ask for a config which cannot be achieved (specifying all 3 variables can lead to invalid
                    # configs.)
                    no_of_samples = 150
                    duration = 0.4
                    computed_timebase = duration / no_of_samples
                    max_timebase_requested = computed_timebase / 4
                    config = TimebaseOptions(max_time_interval=max_timebase_requested,
                                             no_of_samples=no_of_samples,
                                             min_collection_time=duration)
                    timebase_info = device.find_timebase(config)
                except NoValidTimebaseForOptionsError:
                    threw = True
                if not threw:
                    return "didn't throw an NoValidTimebaseForOptionsError."

        self.run_snippet_and_count_problems(drivers_to_use, test)

    def test_get_timebase_throws_on_bad_params(self):
        """test_get_timebase_impossible
        note: test assumes you have set test_helpers.drivers_with_device_connected"""
        if not drivers_with_device_connected:
            return
        drivers_to_use = drivers_with_device_connected

        def test(driver):
            with driver.open_unit() as device:
                threw = False
                try:
                    # we ask for a config which cannot be achieved (specifying all 3 variables can lead to invalid
                    # configs.)
                    no_of_samples = 150
                    duration = 0.4
                    computed_timebase = duration / no_of_samples
                    max_timebase_requested = computed_timebase / 4

                    # we manually invoke the Library method, to check that if invalid options sneak through, we throw
                    # a different error:
                    timebase_info = device.driver.get_timebase(device, 1, no_of_samples)
                except InvalidTimebaseError:
                    threw = True
                if not threw:
                    return "didn't throw an InvalidTimebaseError."

        self.run_snippet_and_count_problems(drivers_to_use, test)


class TimebaseValidationTest(unittest.TestCase):
    def test_valid_config(self):
        request = TimebaseOptions(max_time_interval=0.005,
                                  no_of_samples=None,
                                  min_collection_time=1.)
        actual_timebase = 0.004
        required_max_samples = int(math.ceil(request.min_collection_time / actual_timebase))
        response = TimebaseInfo(timebase_id=7,
                                time_interval=0.004,
                                time_units=None,
                                max_samples=required_max_samples+1,
                                segment_id=0)

        self.assertTrue(Device._validate_timebase(request, response))

    def test_invalid_config(self):
        request = TimebaseOptions(max_time_interval=0.005,
                                  no_of_samples=None,
                                  min_collection_time=1.)
        actual_timebase = 0.004
        required_max_samples = int(math.ceil(request.min_collection_time / actual_timebase))
        response = TimebaseInfo(timebase_id=7,
                                time_interval=0.004,
                                time_units=None,
                                max_samples=required_max_samples-5,
                                segment_id=0)

        self.assertFalse(Device._validate_timebase(request, response))
